@@ -1164,18 +1164,11 @@ void GetTempPath(const size_t maxBytes, char* tmpStr) {
 const char* CSecurityMulti::getTgproTmpPath() {
 	if (tgproTmpPath)
 		return tgproTmpPath;
-	const size_t bufSize = 1024 * 8;
-	char tmpPath[bufSize];
-	const char* tgproTmpSubdir = "tgprotemp";
-	const size_t tgproTmpSubdirLen = strlen(tgproTmpSubdir);
-	const size_t bufAvail = bufSize - tgproTmpSubdirLen - 3;
-	GetTempPath(bufAvail, tmpPath);
-	char* tmpTerminator = tmpPath + strlen(tmpPath);
-	*tmpTerminator++ = PATH_DIVIDER;
-	strcpy(tmpTerminator, tgproTmpSubdir);
-	tmpTerminator += tgproTmpSubdirLen;
-	*tmpTerminator++ = PATH_DIVIDER;
-	*tmpTerminator = '\0';
+	const char* tmpPath = get_tgpro_tmp_path();
+	if (!tmpPath) {
+		vlog.error("%s: Failed to get_tgpro_tmp_path()", __func__);
+		return NULL;
+	}
 #ifdef WIN32
 	if (mkdir(tmpPath) && errno != EEXIST)
 #else
@@ -1204,7 +1197,7 @@ const char* CSecurityMulti::getTgproTmpPath() {
 #endif
 		vlog.error("Failed to create a tgpro temporary directory for autotransfer.");
 
-	return tgproTmpPath = strdup(tmpPath);
+	return tmpPath;
 }
 
 
