@@ -2,7 +2,7 @@
  * Copyright (C) 2011 D. R. Commander.  All Rights Reserved.
  * Copyright 2014-2018 Pierre Ossman for Cendio AB
  * Copyright 2018 Peter Astrand for Cendio AB
- * Copyright (C) 2020-2024 m-privacy GmbH, Berlin
+ * Copyright (C) 2020-2025 m-privacy GmbH, Berlin
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -510,9 +510,7 @@ THREAD_FUNC EncodeManager::pressureMonitorThread(void* param) {
       cpuVal = 0;
       vlog.debug("pressureMonitorThread: failed to open %s", cpuFilename);
     }
-    if (   (saveBandwidthHigh && myself->conn->client.mpCompression == MP_COMPRESSION_AUTO)
-        || mpCompressionSlow(myself->conn->client.mpCompression)
-       ) {
+    if (mpCompressionSlow(myself->conn->client.mpCompression)) {
       if (encoderThreadsMaxUse != encoderThreadsStarted) {
         vlog.debug("pressureMonitorThread: change encoderThreadsMaxUse to %u for mpCompression %u", encoderThreadsStarted, myself->conn->client.mpCompression);
         encoderThreadsMaxUse = encoderThreadsStarted;
@@ -663,6 +661,13 @@ void EncodeManager::pruneLosslessRefresh(const Region& limits)
   MUTEX_UNLOCK(&regionMutex);
 }
 
+void EncodeManager::forceRefresh(const Region& req)
+{
+  lossyRegion.assign_union(req);
+  if (!recentChangeTimer.isStarted())
+    pendingRefreshRegion.assign_union(req);
+}
+
 void EncodeManager::writeUpdate(const UpdateInfo& ui, const PixelBuffer* pb,
                                 const RenderedCursor* renderedCursor)
 {
@@ -729,11 +734,11 @@ void EncodeManager::doUpdate(bool allowLossy, const Region& changed_,
       changed.assign_subtract(renderedCursor->getEffectiveRect());
     }
 
-    if (conn->client.supportsEncoding(pseudoEncodingLastRect))
+    if (likely(conn->client.supportsEncoding(pseudoEncodingLastRect)))
       nRects = 0xFFFF;
     else {
       nRects = 0;
-      if (conn->client.supportsEncoding(encodingCopyRect))
+      if (likely(conn->client.supportsEncoding(encodingCopyRect)))
         nRects += copied.numRects();
       nRects += computeNumRects(changed);
       nRects += computeNumRects(cursorRegion);
@@ -741,14 +746,14 @@ void EncodeManager::doUpdate(bool allowLossy, const Region& changed_,
 
     conn->writer()->writeFramebufferUpdateStart(nRects);
 
-    if (conn->client.supportsEncoding(encodingCopyRect))
+    if (likely(conn->client.supportsEncoding(encodingCopyRect)))
       writeCopyRects(copied, copyDelta);
 
     /*
      * We start by searching for solid rects, which are then removed
      * from the changed region.
      */
-    if (conn->client.supportsEncoding(pseudoEncodingLastRect))
+    if (likely(conn->client.supportsEncoding(pseudoEncodingLastRect)))
       writeSolidRects(&changed, pb);
 
     writeRects(changed, pb);
