@@ -1,7 +1,7 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
  * Copyright (C) 2011 D. R. Commander.  All Rights Reserved.
  * Copyright 2014 Pierre Ossman for Cendio AB
- * Copyright (C) 2015-2024 m-privacy GmbH.  All Rights Reserved.
+ * Copyright (C) 2015-2025 m-privacy GmbH.  All Rights Reserved.
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,9 +49,9 @@ using namespace rfb;
 
 static rfb::LogWriter vlog("MPCompressor");
 
-IntParameter pressureLevelIncrease("PressureLevelIncrease", "CPU pressure above which to raise pressureLevel", 40, 1, 99);
-IntParameter pressureLevelDecrease("PressureLevelDecrease", "CPU pressure below which to reduce pressureLevel", 20, 1, 99);
-IntParameter pressureLevelClear("PressureLevelClear", "CPU pressure below which to reset pressureLevel to 0", 10, 1, 99);
+IntParameter pressureLevelIncrease("PressureLevelIncrease", "CPU pressure above which to raise pressureLevel", 20, 1, 99);
+IntParameter pressureLevelDecrease("PressureLevelDecrease", "CPU pressure below which to reduce pressureLevel", 10, 1, 99);
+IntParameter pressureLevelClear("PressureLevelClear", "CPU pressure below which to reset pressureLevel to 0", 3, 1, 99);
 IntParameter VNCZstdLevel("VNCZstdLevel", "VNC ZSTD compression level (0-22, 0 for ZSTD default (currently 3))", 0, 0, 22);
 BoolParameter MPOptRectSize("MPOptRectSize", "Always try to minimize MP rect sizes", false);
 #if !defined(__APPLE__)
@@ -59,11 +59,14 @@ IntParameter jpegxlEffort("JPEGXLEffort", "JPEGXL normal encoder effort/speed (1
 IntParameter jpegxlDecodingSpeed("JPEGXLDecodingSpeed", "JPEGXL decoding speed (0-4, 0 slowest, best quality, 4 fastest)", 0, 0, 4);
 IntParameter jpegxlDistance("JPEGXLDistance", "JPEGXL distance (0-15, 0 lossless, 15 fastest)", 1, 0, 15);
 #if !defined(WIN32)
-IntParameter autoLbJpegxlRate("AutoLbJpegxlRate", "Rate in n/10 to mix JPEGXL under JPEGLB with Auto compression in Low Bandwidth mode (0-10, 0 none, 10 all)", 0, 0, 10);
+IntParameter autoLbH264Rate("AutoLbH264Rate", "Rate in n/10 to mix H264 under JPEGLB in Save Bandwidth High mode (0-10, 0 none, 10 all)", 10, 0, 10);
+IntParameter autoLbH264MinSizeK("AutoLbH264MinSizeK", "Min Rect size for H264 in Save Bandwidth High mode in KPixel (0-999)", 10, 0, 999);
 BoolParameter jpegxlMultiThread("JPEGXLMultiThread", "Enable internal multi threading in libjxl on top of our own multi threading", false);
 #endif
 #endif
 
+#define MAXPRESSURELEVEL 10
+
 struct TightMPJpegConfiguration {
 	int quality;
 };
@@ -112,8 +115,12 @@ void MPCompressor::setCpuPressure(rdr::U8 value)
   vlog.verbose("setCpuPressure(): got value %u", value);
   cpuPressure = value;
   if (value > pressureLevelIncrease) {
-    pressureLevel++;
-    vlog.debug("setCpuPressure(): value %u above %s, increase pressureLevel to %u", value, pressureLevelIncrease.getValueStr(), pressureLevel);
+    if (pressureLevel < MAXPRESSURELEVEL) {
+      pressureLevel++;
+      vlog.debug("setCpuPressure(): value %u above %s, increase pressureLevel to %u", value, pressureLevelIncrease.getValueStr(), pressureLevel);
+    } else {
+      vlog.debug("setCpuPressure(): value %u above %s, but pressureLevel already at max %u", value, pressureLevelIncrease.getValueStr(), MAXPRESSURELEVEL);
+    }
   } else if (pressureLevel > 0) {
     if (value < pressureLevelClear) {
       pressureLevel = 0;
@@ -296,7 +303,7 @@ MPCompressor::MPCompressor(int bufferLen) : MemOutStream(bufferLen),
 	snprintf(jpegErr->lastError, JMSG_LENGTH_MAX, "No error");
 	jpegErr->pub.error_exit = JpegErrorExit;
 	jpegErr->pub.output_message = JpegOutputMessage;
-	if(setjmp(jpegErr->jmpBuffer)) {
+	if(unlikely(setjmp(jpegErr->jmpBuffer))) {
 		// this will execute if libjpeg has an error
 		throw rdr::Exception("%s", jpegErr->lastError);
 	}
@@ -394,11 +401,13 @@ static inline int i420DataSize(int mpWidth, int mpHeight)
 	return mpWidth * mpHeight * 3 / 2 + 2 * ((mpWidth & 1) * mpHeight + (mpHeight & 1) * mpWidth);
 }
 
+#if !defined(WIN32)
 bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 	const PixelFormat& pf, int mpLevel, int mpCompression)
 {
-	int w = r.width();
-	int h = r.height();
+	const int w = r.width();
+	const int h = r.height();
+	const int rectSize = w * h;
 	int mpWidth;
 	int mpHeight;
 //	const int mpCompressionWanted = mpCompression;
@@ -428,41 +437,42 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 	if (stride == 0)
 		stride = w;
 
-	if (mpLevel < 0 || mpLevel > 9) {
+	if (unlikely(mpLevel < 0 || mpLevel > 9)) {
 		vlog.error("invalid mpLevel %i, setting to 9", mpLevel);
 		mpLevel = 9;
 	}
 
 	if (mpCompression == MP_COMPRESSION_AUTO) {
 		if (saveBandwidthHigh) {
-			if (pressureLevel == 0) {
-				mpCompression = MP_COMPRESSION_H264;
-			} else {
+			if (rectSize < autoLbH264MinSizeK * 1000) {
 				mpCompression = MP_COMPRESSION_JPEGLB;
-			}
-		} else if (saveBandwidth) {
-#if !defined(__APPLE__) && !defined(WIN32)
-			if (clientSupportsTightMPJpegXL && autoLbJpegxlRate > 0) {
-				rdr::U8 randByte = 0;
-				int autoLbJpegxlRateUsed = autoLbJpegxlRate;
+			} else {
+				int autoLbH264RateUsed = autoLbH264Rate;
 
-				if (pressureLevel > 0 && autoLbJpegxlRateUsed >= pressureLevel)
-					autoLbJpegxlRateUsed -= pressureLevel;
-				getrandom(&randByte, 1, 0);
-				if ((int) randByte * 10 / 256 >= autoLbJpegxlRateUsed) {
-					vlog.verbose("AUTO saveBandwidth: w: %u, h: %u, lbrate %u, randByte %u, choosing JPEGLB", w, h, autoLbJpegxlRateUsed, randByte);
+				if (pressureLevel > 0) {
+					if (autoLbH264RateUsed > pressureLevel)
+						autoLbH264RateUsed -= pressureLevel;
+					else
+						autoLbH264RateUsed = 0;
+				}
+				if (autoLbH264RateUsed == 10) {
+					mpCompression = MP_COMPRESSION_H264;
+				} else if (autoLbH264RateUsed == 0) {
 					mpCompression = MP_COMPRESSION_JPEGLB;
 				} else {
-					vlog.verbose("AUTO saveBandwidth: w: %u, h: %u, lbrate %u, randByte %u, choosing JPEGXL", w, h, autoLbJpegxlRateUsed, randByte);
-					mpCompression = MP_COMPRESSION_JPEGXL;
+					rdr::U8 randByte = 0;
+
+					getrandom(&randByte, 1, 0);
+					if ((int) randByte * 10 / 256 < autoLbH264RateUsed) {
+						mpCompression = MP_COMPRESSION_H264;
+					} else {
+						mpCompression = MP_COMPRESSION_JPEGLB;
+					}
 				}
-			} else {
-#endif
-//				vlog.verbose("AUTO saveBandwidth: w: %u, h: %u, autoLbJpegxlRate %u, choosing JPEGLB", w, h, (int) autoLbJpegxlRate);
-				mpCompression = MP_COMPRESSION_JPEGLB;
-#if !defined(__APPLE__) && !defined(WIN32)
 			}
-#endif
+		} else if (saveBandwidth) {
+//			vlog.verbose("AUTO saveBandwidth: w: %u, h: %u, choosing JPEGLB", w, h);
+			mpCompression = MP_COMPRESSION_JPEGLB;
 		} else {
 			if (pressureLevel == 0)
 				mpCompression = MP_COMPRESSION_JPEG;
@@ -486,7 +496,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 	}
 	if (mpCompression == MP_COMPRESSION_JPEGLB) {
 		mpCompression = MP_COMPRESSION_JPEG;
-		if (mpLevel > 0)
+		if (likely(mpLevel > 0))
 			mpLevel--;
 	}
 	/* do not waste resources for compression of small rects,
@@ -495,16 +505,16 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 	/* JPEG has min usedSize of 640B, try to save time and space (4B/pixel) */
 	if (mpCompression == MP_COMPRESSION_JPEG) {
 		if (mpLevel == 9) {
-			if (w * h <= 20)
+			if (rectSize <= 20)
 				mpCompression = MP_COMPRESSION_RAW;
-			else if (w * h <= 200)
+			else if (rectSize <= 200)
 				mpCompression = MP_COMPRESSION_ZSTD;
 			else
 				wantOptSize = true;
 		} else {
-			if (w * h <= 12)
+			if (rectSize <= 12)
 				mpCompression = MP_COMPRESSION_RAWI420;
-			else if (w * h <= 400)
+			else if (rectSize <= 400)
 				mpCompression = MP_COMPRESSION_I420ZSTD;
 			else
 				wantOptSize = true;
@@ -513,11 +523,11 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 	/* JPEGXL has min usedSize of 88B, but small sizes are inefficient: */
 	/* try to optimize time and space (4B/pixel) */
 	} else if (mpCompression == MP_COMPRESSION_JPEGXL) {
-		if (w * h <= 20)
+		if (rectSize <= 20)
 			mpCompression = MP_COMPRESSION_RAW;
-		else if (mpLevel < 9 && (h < 4 || w * h < 200))
+		else if (mpLevel < 9 && (h < 4 || rectSize < 200))
 			mpCompression = MP_COMPRESSION_I420ZSTD;
-		else if (h < 4 || w * h <= 100)
+		else if (h < 4 || rectSize <= 100)
 			mpCompression = MP_COMPRESSION_ZSTD;
 		else
 			wantOptSize = true;
@@ -538,7 +548,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		mpPixelsize = 4;
 		mpStride = mpWidth * mpPixelsize;
 		dataSize = mpHeight * mpStride;
-		if (is888) {
+		if (likely(is888)) {
 //			vlog.verbose("RAW: directly use OutStream buffer with size %u", dataSize);
 			direct = true;
 			data = getptr(dataSize);
@@ -560,7 +570,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		mpStride = mpWidth * mpPixelsize;
 //		vlog.verbose("JPEG: w: %u, h: %u, stride: %u, mpWidth: %u, mpStride: %u", w, h, stride, mpWidth, mpStride);
 		dataSize = mpHeight * mpStride;
-		if(setjmp(jpegErr->jmpBuffer)) {
+		if(unlikely(setjmp(jpegErr->jmpBuffer))) {
 			// this will execute if libjpeg has an error
 			jpeg_abort_compress(jpegCinfo);
 			throw rdr::Exception("%s", jpegErr->lastError);
@@ -590,7 +600,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 			jpegCinfo->comp_info[0].v_samp_factor = 1;
 		}
 #ifdef JCS_EXTENSIONS
-		if (is888) {
+		if (likely(is888)) {
 			JSAMPROW *jpegRowPointer;
 
 			mpStride = stride * srcPixelsize;
@@ -616,7 +626,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 					return compress(buf, stride, r, pf, mpLevel, MP_COMPRESSION_I420ZSTD);
 				}
 			} else {
-				const int zstdEstSize = w * h * 2;
+				const int zstdEstSize = rectSize * 2;
 
 				if (usedSize > zstdEstSize + MP_COMPRESSION_MIN_SAVE_SIZE) {
 					vlog.verbose("optimize: mpCompression: %s, mpLevel: %u, w: %u, h: %u, stride: %u, mpWidth: %u, usedSize %lu > zstdEstSize %u + %u, repeat with ZSTD", mpCompressionName(mpCompression), mpLevel, w, h, stride, mpWidth, usedSize, zstdEstSize, MP_COMPRESSION_MIN_SAVE_SIZE);
@@ -629,9 +639,9 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 				compressionCount[mpCompression]++;
 				totalBytes[mpCompression] += mpWidth * mpHeight * srcPixelsize;
 				totalCBytes[mpCompression] += usedSize;
-				if (usedSize < minCompressed[mpCompression])
+				if (unlikely(usedSize < minCompressed[mpCompression]))
 					minCompressed[mpCompression] = usedSize;
-				if (usedSize > maxCompressed[mpCompression])
+				if (unlikely(usedSize > maxCompressed[mpCompression]))
 					maxCompressed[mpCompression] = usedSize;
 			}
 			return true;
@@ -639,7 +649,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 #endif
 //		vlog.verbose2("JPEG indirect: w: %u, h: %u, stride: %u, mpWidth: %u, mpStride: %u", w, h, stride, mpWidth, mpStride);
 		data = (rdr::U8*) malloc(dataSize);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("malloc(%u) for JPEG failed", dataSize);
 			return false;
 		}
@@ -651,7 +661,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 			jpegxlStatus = JxlEncoderSetParallelRunner(jpegxlEnc,
 					JxlThreadParallelRunner,
 					jpegxlRunner);
-			if (JXL_ENC_SUCCESS != jpegxlStatus) {
+			if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 				vlog.error("JxlEncoderSetParallelRunner() failed with error %i", JxlEncoderGetError(jpegxlEnc));
 				return false;
 			}
@@ -662,40 +672,40 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		jpegxlBasicInfo.intrinsic_xsize = mpWidth;
 		jpegxlBasicInfo.intrinsic_ysize = mpHeight;
 		jpegxlStatus = JxlEncoderSetBasicInfo(jpegxlEnc, &jpegxlBasicInfo);
-		if (JXL_ENC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JxlEncoderSetBasicInfo() failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			return false;
 		}
 		jpegxlStatus = JxlEncoderSetColorEncoding(jpegxlEnc, &jpegxlColorEncoding);
-		if (JXL_ENC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JxlEncoderSetColorEncoding() failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			return false;
 		}
 		jpegxlFrameSettings = JxlEncoderFrameSettingsCreate(jpegxlEnc, nullptr);
 		jpegxlStatus = JxlEncoderSetFrameDistance(jpegxlFrameSettings, jpegxlDistance);
-		if (JXL_ENC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JxlEncoderSetFrameDistance() failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			return false;
 		}
 		jpegxlStatus = JxlEncoderFrameSettingsSetOption(jpegxlFrameSettings, JXL_ENC_FRAME_SETTING_EFFORT, jpegxlEffortUsed);
-		if (JXL_ENC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JxlEncoderFrameSettingsSetOption(JXL_ENC_FRAME_SETTING_EFFORT) failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			return false;
 		}
 		jpegxlStatus = JxlEncoderFrameSettingsSetOption(jpegxlFrameSettings, JXL_ENC_FRAME_SETTING_DECODING_SPEED, jpegxlDecodingSpeedUsed);
-		if (JXL_ENC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JxlEncoderFrameSettingsSetOption(JXL_ENC_FRAME_SETTING_DECODING_SPEED) failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			return false;
 		}
 //		vlog.debug("JPEGXL: speed_tier: %u, decoding_speed_tier: %u, distance: %.1f", (int) jpegxlFrameSettings->values.cparams.speed_tier, (int) jpegxlFrameSettings->values.cparams.decoding_speed_tier, (float) jpegxlFrameSettings->values.cparams.butteraugli_distance);
 #if 0 // this triggers a double free bug in libjxl
 		jpegxlStatus = JxlEncoderFrameSettingsSetOption(jpegxlFrameSettings, JXL_ENC_FRAME_SETTING_EXTRA_CHANNEL_RESAMPLING, 8);
-		if (JXL_ENC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JxlEncoderFrameSettingsSetOption(JXL_ENC_FRAME_SETTING_EXTRA_CHANNEL_RESAMPLING) failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			return false;
 		}
 #endif
-		if (is888) {
+		if (likely(is888)) {
 			dataSize = stride * srcPixelsize * h;
 //			vlog.verbose2("JPEGXL: direct: w: %u, h: %u, stride: %u, dataSize: %u, effort: %u, decoding speed: %u, distance: %u", w, h, stride, dataSize, jpegxlEffortUsed, jpegxlDecodingSpeedUsed, (int) jpegxlDistance);
 			/* last param = data align in bytes, 0 = not needed, might be stride */
@@ -709,7 +719,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 				&jpegxlPixelFormat,
 				(void*) buf,
 				dataSize);
-			if (JXL_ENC_SUCCESS != jpegxlStatus) {
+			if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 				vlog.error("JxlEncoderAddImageFrame() failed with error %i", JxlEncoderGetError(jpegxlEnc));
 				return false;
 			}
@@ -727,7 +737,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 					nextOut = startOut + offset;
 					availOut = avail() - offset;
 					vlog.debug("JPEGXL: needed more bytes, increased to avail of %lu", availOut);
-					if (availOut < 2048) {
+					if (unlikely(availOut < 2048)) {
 						vlog.error("failed to increase buffer!");
 						return false;
 					}
@@ -749,7 +759,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 					return compress(buf, stride, r, pf, mpLevel, MP_COMPRESSION_I420ZSTD);
 				}
 			} else {
-				const int zstdEstSize = w * h * 2;
+				const int zstdEstSize = rectSize * 2;
 
 				if (usedSize > zstdEstSize + MP_COMPRESSION_MIN_SAVE_SIZE) {
 					vlog.verbose("optimize: mpCompression: %s, mpLevel: %u, w: %u, h: %u, stride: %u, mpWidth: %u, usedSize %lu > zstdEstSize %u + %u, repeat with ZSTD", mpCompressionName(mpCompression), mpLevel, w, h, stride, mpWidth, usedSize, zstdEstSize, MP_COMPRESSION_MIN_SAVE_SIZE);
@@ -762,9 +772,9 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 				compressionCount[mpCompression]++;
 				totalBytes[mpCompression] += mpWidth * mpHeight * srcPixelsize;
 				totalCBytes[mpCompression] += usedSize;
-				if (usedSize < minCompressed[mpCompression])
+				if (unlikely(usedSize < minCompressed[mpCompression]))
 					minCompressed[mpCompression] = usedSize;
-				if (usedSize > maxCompressed[mpCompression])
+				if (unlikely(usedSize > maxCompressed[mpCompression]))
 					maxCompressed[mpCompression] = usedSize;
 			}
 			return true;
@@ -782,7 +792,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		jpegxlPixelFormat.align = (size_t) 0;
 		vlog.debug("JPEGXL: converting: w: %u, h: %u, stride: %u, mpWidth: %u, mpStride: %u", w, h, stride, mpWidth, mpStride);
 		data = (rdr::U8*) malloc(dataSize);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("malloc(%u) for JPEGXL failed", dataSize);
 			return false;
 		}
@@ -802,7 +812,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		/* mpLevel setting LOW, MEDIUM, HIGH, better is slower */
 		if (mpLevel >= 7)
 			welsComplexity = HIGH_COMPLEXITY;
-		else if (mpLevel <= 3)
+		else if (unlikely(mpLevel <= 3))
 			welsComplexity = LOW_COMPLEXITY;
 		else
 			welsComplexity = MEDIUM_COMPLEXITY;
@@ -814,12 +824,12 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		 */
 		dataSize = i420DataSize(mpWidth, mpHeight);
 		data = (rdr::U8*) malloc(dataSize);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("malloc(%u) for H264 I420 data failed", dataSize);
 			return false;
 		}
 		rv = WelsCreateSVCEncoder(&welsEncoder);
-		if (rv != cmResultSuccess || welsEncoder == NULL) {
+		if (unlikely(rv != cmResultSuccess || welsEncoder == NULL)) {
 			vlog.error("WelsCreateSVCEncoder() failed with error %i", rv);
 			free(data);
 			return false;
@@ -827,14 +837,14 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		welsParam.iPicWidth = mpWidth;
 		welsParam.iPicHeight = mpHeight;
 		rv = welsEncoder->Initialize(&welsParam);
-		if (rv != cmResultSuccess) {
+		if (unlikely(rv != cmResultSuccess)) {
 			vlog.error("welsEncoder->Initialize() failed with error %i", rv);
 			free(data);
 			WelsDestroySVCEncoder(welsEncoder);
 			return false;
 		}
 		rv = welsEncoder->SetOption(ENCODER_OPTION_DATAFORMAT, &welsVideoFormat);
-		if (rv != cmResultSuccess) {
+		if (unlikely(rv != cmResultSuccess)) {
 			vlog.error("elsEncoder->SetOption(ENCODER_OPTION_DATAFORMAT) failed with error %i", rv);
 			free(data);
 			welsEncoder->Uninitialize();
@@ -842,7 +852,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 			return false;
 		}
 		rv = welsEncoder->SetOption(ENCODER_OPTION_COMPLEXITY, &welsComplexity);
-		if (rv != cmResultSuccess) {
+		if (unlikely(rv != cmResultSuccess)) {
 			vlog.error("elsEncoder->SetOption(ENCODER_OPTION_COMPLEXITY) failed with error %i", rv);
 			free(data);
 			welsEncoder->Uninitialize();
@@ -850,7 +860,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 			return false;
 		}
 		rv = welsEncoder->SetOption(ENCODER_OPTION_SVC_ENCODE_PARAM_BASE, &welsParam);
-		if (rv != cmResultSuccess) {
+		if (unlikely(rv != cmResultSuccess)) {
 			vlog.error("elsEncoder->SetOption(ENCODER_OPTION_SVC_ENCODE_PARAM_BASE) failed with error %i", rv);
 			free(data);
 			welsEncoder->Uninitialize();
@@ -866,13 +876,13 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		 * uneven mpWidth and extra line for uneven mpHeight
 		 */
 		dataSize = i420DataSize(mpWidth, mpHeight);
-		if (is888) {
+		if (likely(is888)) {
 //			vlog.verbose("RAWI420: directly use OutStream buffer with size %u", dataSize);
 			direct = true;
 			data = getptr(dataSize);
 		} else {
 			data = (rdr::U8*) malloc(dataSize);
-			if (!data) {
+			if (unlikely(!data)) {
 				vlog.error("malloc(%u) for RAWI420 failed", dataSize);
 				return false;
 			}
@@ -881,15 +891,15 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		mpPixelsize = 4;
 		mpStride = mpWidth * mpPixelsize;
 		dataSize = mpHeight * mpStride;
-		if (!zstdCctx) {
+		if (unlikely(!zstdCctx)) {
 			zstdCctx = ZSTD_createCCtx();
-			if (!zstdCctx) {
+			if (unlikely(!zstdCctx)) {
 				vlog.error("ZSTD_createCCtx() failed");
 				return false;
 			}
 		}
 		data = (rdr::U8*) malloc(dataSize);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("malloc(%u) for ZSTD failed", dataSize);
 			return false;
 		}
@@ -899,9 +909,9 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		mpPixelsize = 3;
 		mpStride = mpWidth * mpPixelsize;
 //		vlog.verbose("I420ZSTD: w: %u, h: %u, stride: %u, mpWidth: %u, mpStride: %u", w, h, stride, mpWidth, mpStride);
-		if (!zstdCctx) {
+		if (unlikely(!zstdCctx)) {
 			zstdCctx = ZSTD_createCCtx();
-			if (!zstdCctx) {
+			if (unlikely(!zstdCctx)) {
 				vlog.error("ZSTD_createCCtx() failed");
 				return false;
 			}
@@ -911,7 +921,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		 */
 		dataSize = i420DataSize(mpWidth, mpHeight);
 		data = (rdr::U8*) malloc(dataSize);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("malloc(%u) for I420ZSTD I420 data failed", dataSize);
 			return false;
 		}
@@ -919,10 +929,10 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		vlog.error("invalid MP compression %u", mpCompression);
 		return false;
 	}
-	if (!is888) {
+	if (unlikely(!is888)) {
 		vlog.verbose("not is888, use tmpBuf: w: %u, h: %u, stride: %u, mpWidth: %u, mpStride: %u", w, h, stride, mpWidth, mpStride);
-		tmpBuf = (rdr::U8*) malloc(w * h * srcPixelsize);
-		if (!tmpBuf) {
+		tmpBuf = (rdr::U8*) malloc(rectSize * srcPixelsize);
+		if (unlikely(!tmpBuf)) {
 			vlog.error("malloc for tmpBuf failed");
 			if (mpCompression == MP_COMPRESSION_RAW) {
 				if (!direct)
@@ -1045,7 +1055,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 	} else {
 		if (mpCompression == MP_COMPRESSION_RAW) {
 //			vlog.verbose("direct use: wanted compression: %u, selected compression: %u, mpLevel: %u, w: %u, h: %u, stride: %u, mpStride: %u", mpCompressionWanted, mpCompression, mpLevel, w, h, stride, mpStride);
-			if (direct) {
+			if (likely(direct)) {
 				memcpy(data, buf, dataSize);
 			} else {
 				free(data);
@@ -1056,14 +1066,14 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 			if (needI420) {
 				rgbToI420(data, (rdr::U8*) buf, w, h, srcPixelsize);
 			} else {
-				memcpy(data, buf, w * h * srcPixelsize);
+				memcpy(data, buf, rectSize * srcPixelsize);
 			}
 		}
 	}
 
 	/* write out: body */
 	if (mpCompression == MP_COMPRESSION_RAW) {
-		if (direct) {
+		if (likely(direct)) {
 			reposition(length() + dataSize);
 		} else {
 			writeBytes(data, dataSize);
@@ -1090,7 +1100,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 			&jpegxlPixelFormat,
 			(void*) data,
 			dataSize);
-		if (JXL_ENC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JxlEncoderAddImageFrame() failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			free(data);
 			if (tmpBuf)
@@ -1111,7 +1121,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 				nextOut = startOut + offset;
 				availOut = avail() - offset;
 				vlog.debug("JPEGXL: needed more bytes, increased to avail of %lu", availOut);
-				if (availOut < 2048) {
+				if (unlikely(availOut < 2048)) {
 					vlog.error("failed to increase buffer!");
 					if (!direct)
 						free(data);
@@ -1122,7 +1132,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 			}
 		}
 		free(data);
-		if (JXL_ENC_SUCCESS != jpegxlProcessResult) {
+		if (unlikely(JXL_ENC_SUCCESS != jpegxlProcessResult)) {
 			vlog.error("JxlEncoderProcessOutput() failed with error %i", JxlEncoderGetError(jpegxlEnc));
 			if (tmpBuf)
 				free(tmpBuf);
@@ -1154,7 +1164,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		memset (&info, 0, sizeof(info));
 		rv = welsEncoder->EncodeFrame(&welsPic, &info);
 		free(data);
-		if (rv == cmResultSuccess && info.eFrameType != videoFrameTypeInvalid && info.eFrameType != videoFrameTypeSkip && info.iLayerNum > 0) {
+		if (likely(rv == cmResultSuccess && info.eFrameType != videoFrameTypeInvalid && info.eFrameType != videoFrameTypeSkip && info.iLayerNum > 0)) {
 			for (i = 0; i < info.iLayerNum; i++) {
 				pLayer = &info.sLayerInfo[i];
 				iCurLayerBits = 0;
@@ -1174,7 +1184,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		welsEncoder->Uninitialize();
 		WelsDestroySVCEncoder(welsEncoder);
 	} else if (mpCompression == MP_COMPRESSION_RAWI420) {
-		if (direct) {
+		if (likely(direct)) {
 			reposition(length() + dataSize);
 		} else {
 			writeBytes(data, dataSize);
@@ -1185,7 +1195,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		int compressedSize;
 
 		compressedSize = ZSTD_compressCCtx(zstdCctx, getptr(compressedDataSize), compressedDataSize, data, dataSize, VNCZstdLevel);
-		if (compressedSize > 0) {
+		if (likely(compressedSize > 0)) {
 			reposition(length() + compressedSize);
 		} else {
 			vlog.error("ZSTD: ZSTD_compressCCtx() failed with error %i", compressedSize);
@@ -1200,7 +1210,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 		int compressedSize;
 
 		compressedSize = ZSTD_compressCCtx(zstdCctx, getptr(compressedDataSize), compressedDataSize, data, dataSize, VNCZstdLevel);
-		if (compressedSize > 0) {
+		if (likely(compressedSize > 0)) {
 			reposition(length() + compressedSize);
 		} else {
 			vlog.error("I420+ZSTD: ZSTD_compressCctx() failed with error %i", compressedSize);
@@ -1229,7 +1239,7 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 				return compress(buf, stride, r, pf, mpLevel, MP_COMPRESSION_I420ZSTD);
 			}
 		} else {
-			const int rawSize = w * h * 4;
+			const int rawSize = rectSize * 4;
 
 			if (rawSize < 80 || mpCompression == MP_COMPRESSION_ZSTD || mpCompression == MP_COMPRESSION_I420ZSTD) {
 				if (usedSize > rawSize + MP_COMPRESSION_MIN_SAVE_SIZE) {
@@ -1260,3 +1270,4 @@ bool MPCompressor::compress(const rdr::U8 *buf, int stride, const Rect& r,
 	}
 	return true;
 }
+#endif /* #if !defined(WIN32) */
