@@ -2,7 +2,7 @@
  * Copyright (C) 2004-2005 Cendio AB. All rights reserved.
  * Copyright (C) 2011 D. R. Commander.  All Rights Reserved.
  * Copyright 2014 Pierre Ossman for Cendio AB
- * Copyright (C) 2020-2024 m-privacy GmbH.  All Rights Reserved.
+ * Copyright (C) 2020-2025 m-privacy GmbH.  All Rights Reserved.
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,7 +89,7 @@ JpegSkipInputData(j_decompress_ptr jpegDinfo, long num_bytes)
 {
   MP_JPEG_SRC_MGR *jpegSrc = (MP_JPEG_SRC_MGR *)jpegDinfo->src;
 
-  if (num_bytes < 0 || (size_t)num_bytes > jpegSrc->pub.bytes_in_buffer) {
+  if (unlikely(num_bytes < 0 || (size_t)num_bytes > jpegSrc->pub.bytes_in_buffer)) {
     ERREXIT(jpegDinfo, JERR_BUFFER_SIZE);
   } else {
     jpegSrc->pub.next_input_byte += (size_t) num_bytes;
@@ -238,14 +238,14 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 	compressionCountLocal[mpCompression]++;
 	totalBytes[mpCompression] += w * h * targetBufPixelsize;
 	totalCBytes[mpCompression] += srcBufLen;
-	if (srcBufLen < minCompressed[mpCompression])
+	if (unlikely(srcBufLen < minCompressed[mpCompression]))
 		minCompressed[mpCompression] = srcBufLen;
-	if (srcBufLen > maxCompressed[mpCompression])
+	if (unlikely(srcBufLen > maxCompressed[mpCompression]))
 		maxCompressed[mpCompression] = srcBufLen;
 
 	/* solid? */
 	if (mpCompression == MP_COMPRESSION_SOLID) {
-		if (is888) {
+		if (likely(is888)) {
 			rdr::U8 * line;
 
 			vlog.verbose2("SOLID direct: x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, raw size: %u, compressed: %u, colour RGBA: %#02X%02X%02X%02X", r.tl.x, r.tl.y, w, h, targetBufPixelsize, stride, h * w * targetBufPixelsize, srcBufLen, *srcBuf, *(srcBuf+1), *(srcBuf+2), *(srcBuf+3));
@@ -263,7 +263,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 
 			vlog.verbose2("SOLID buffered: x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, raw size: %u, compressed: %u, colour RGBA: %#02X%02X%02X%02X", r.tl.x, r.tl.y, w, h, targetBufPixelsize, stride, h * w * targetBufPixelsize, srcBufLen, *srcBuf, *(srcBuf+1), *(srcBuf+2), *(srcBuf+3));
 			data = (rdr::U8 *) malloc(w * h * targetBufPixelsize);
-			if (!data) {
+			if (unlikely(!data)) {
 				vlog.error("SOLID: failed to alloc memory for RGBA data");
 				return;
 			}
@@ -296,7 +296,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		mpWidth = w;
 		mpHeight = h;
 #ifdef JCS_EXTENSIONS
-		if (is888)
+		if (likely(is888))
 			direct = true;
 		mpPixelsize = 4;
 #else
@@ -311,7 +311,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		snprintf(jpegErr->lastError, JMSG_LENGTH_MAX, "No error");
 		jpegErr->pub.error_exit = JpegErrorExit;
 		jpegErr->pub.output_message = JpegOutputMessage;
-		if(setjmp(jpegErr->jmpBuffer)) {
+		if(unlikely(setjmp(jpegErr->jmpBuffer))) {
 			// this will execute if libjpeg has an error
 			throw rdr::Exception("%s", jpegErr->lastError);
 		}
@@ -324,7 +324,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		jpegSrc->pub.term_source = JpegNoOp;
 		jpegSrc->instance = this;
 		jpegDinfo->src = (struct jpeg_source_mgr *)jpegSrc;
-		if(setjmp(jpegErr->jmpBuffer)) {
+		if(unlikely(setjmp(jpegErr->jmpBuffer))) {
 			// this will execute if libjpeg has an error
 			jpeg_abort_decompress(jpegDinfo);
 			if (jpegRowPointer) delete[] jpegRowPointer;
@@ -340,14 +340,14 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 #endif
 		jpegRowPointer = new JSAMPROW[mpHeight];
 #ifdef JCS_EXTENSIONS
-		if (direct) {
+		if (likely(direct)) {
 			vlog.verbose2("direct: compression: %u (%s), mpLevel: %u, x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpStride: %u, raw size: %u, compressed: %u", mpCompression, mpCompressionName(mpCompression), mpLevel, r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpStride, h * w * mpPixelsize, srcBufLen);
 			mpStride = stride * targetBufPixelsize;
 			data = buf;
 		} else {
 #endif
 			data = (rdr::U8 *) malloc(mpHeight * mpStride);
-			if (!data) {
+			if (unlikely(!data)) {
 				vlog.error("JPEG: failed to alloc memory for RGBA data");
 				delete[] jpegRowPointer;
 				return;
@@ -359,9 +359,11 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		for (int y = 0; y < mpHeight; y++)
 			jpegRowPointer[y] = (JSAMPROW) data + y * mpStride;
 		jpeg_start_decompress(jpegDinfo);
-		if (   jpegDinfo->output_width != (unsigned) mpWidth
-		    || jpegDinfo->output_height != (unsigned) mpHeight
-		    || jpegDinfo->output_components != mpPixelsize) {
+		if (unlikely(   jpegDinfo->output_width != (unsigned) mpWidth
+			     || jpegDinfo->output_height != (unsigned) mpHeight
+			     || jpegDinfo->output_components != mpPixelsize
+			    )
+		   ) {
 			jpeg_abort_decompress(jpegDinfo);
 			if (jpegRowPointer) delete[] jpegRowPointer;
 			vlog.debug("rect size mismatch: expected %ux%ux%u, got MP image with %ux%ux%u", mpWidth, mpHeight, mpPixelsize, jpegDinfo->output_width, jpegDinfo->output_height, jpegDinfo->output_components);
@@ -373,7 +375,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		}
 		jpeg_finish_decompress(jpegDinfo);
 		delete[] jpegRowPointer;
-		if(setjmp(jpegErr->jmpBuffer)) {
+		if(unlikely(setjmp(jpegErr->jmpBuffer))) {
 			// this will execute if libjpeg has an error
 			return;
 		}
@@ -382,7 +384,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		delete jpegSrc;
 		delete jpegDinfo;
 #ifdef JCS_EXTENSIONS
-		if (direct)
+		if (likely(direct))
 			return;
 #endif
 #if !defined(__APPLE__)
@@ -396,20 +398,20 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 
 		JxlDecoderReset(jpegxlDecoder);
 		jpegxlStatus = JxlDecoderSubscribeEvents(jpegxlDecoder, JXL_DEC_BASIC_INFO | JXL_DEC_FULL_IMAGE);
-		if (JXL_DEC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_DEC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JPEGXL: JxlDecoderSubscribeEvents() failed with status %i", jpegxlStatus);
 			return;
 		}
 #ifndef WIN32
 		jpegxlStatus = JxlDecoderSetParallelRunner(jpegxlDecoder, JxlResizableParallelRunner, jpegxlRunner);
-		if (JXL_DEC_SUCCESS != jpegxlStatus) {
+		if (unlikely(JXL_DEC_SUCCESS != jpegxlStatus)) {
 			vlog.error("JPEGXL: JxlDecoderSetParallelRunner() failed with status %i", jpegxlStatus);
 			return;
 		}
 #endif
 		JxlDecoderSetInput(jpegxlDecoder, srcBuf, srcBufLen);
 		JxlDecoderCloseInput(jpegxlDecoder);
-		if (is888) {
+		if (likely(is888)) {
 			direct = true;
 			data = buf;
 			vlog.verbose2("JPEGXL: direct: thread: %lu, x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpWidth: %u, mpHeight: %u, mpStride: %u, raw size: %u, compressed: %u", threadNum, r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpWidth, mpHeight, mpStride, h * w * mpPixelsize, srcBufLen);
@@ -424,7 +426,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 				case JXL_DEC_BASIC_INFO:
 					JxlBasicInfo info;
 					jpegxlStatus = JxlDecoderGetBasicInfo(jpegxlDecoder, &info);
-					if (JXL_DEC_SUCCESS != jpegxlStatus) {
+					if (unlikely(JXL_DEC_SUCCESS != jpegxlStatus)) {
 						vlog.error("JPEGXL: JxlDecoderGetBasicInfo() failed");
 						if (data && data != buf)
 							free(data);
@@ -432,8 +434,9 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 					}
 					mpWidth = info.xsize;
 					mpHeight = info.ysize;
-					if (   (mpWidth != w)
-					    || (mpHeight != h)
+					if (unlikely(   (mpWidth != w)
+						     || (mpHeight != h)
+						    )
 					   ) {
 						vlog.error("JPEGXL: rect size mismatch: expected %ux%u, got MP image with %ux%u", w, h, mpWidth, mpHeight);
 						if (data && data != buf)
@@ -450,7 +453,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 					size_t buffer_size;
 					JxlPixelFormat format;
 
-					if (direct) {
+					if (likely(direct)) {
 						if (data != buf) {
 							free(data);
 							vlog.error("JPEGXL: data already allocated, repeated JXL_DEC_NEED_IMAGE_OUT_BUFFER!");
@@ -461,7 +464,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 						format = {(uint32_t) targetBufPixelsize, JXL_TYPE_UINT8, JXL_NATIVE_ENDIAN, (size_t) mpStride};
 					} else {
 						data = (rdr::U8 *) malloc(mpHeight * mpStride);
-						if (!data) {
+						if (unlikely(!data)) {
 							vlog.error("JPEGXL: failed to alloc memory for RGBA data");
 							return;
 						}
@@ -469,20 +472,20 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 						format = {(uint32_t) targetBufPixelsize, JXL_TYPE_UINT8, JXL_NATIVE_ENDIAN, 0};
 					}
 					jpegxlStatus = JxlDecoderImageOutBufferSize(jpegxlDecoder, &format, &buffer_size);
-					if (JXL_DEC_SUCCESS != jpegxlStatus) {
+					if (unlikely(JXL_DEC_SUCCESS != jpegxlStatus)) {
 						vlog.error("JPEGXL: JxlDecoderImageOutBufferSize() failed");
 						if (data != buf)
 							free(data);
 						return;
 					}
-					if (buffer_size != (size_t) mpHeight * mpStride) {
+					if (unlikely(buffer_size != (size_t) mpHeight * mpStride)) {
 						vlog.error("JPEGXL: image size mismatch: got size %lu, wanted size %u", buffer_size, mpHeight * mpStride);
 						if (data != buf)
 							free(data);
 						return;
 					}
 					jpegxlStatus = JxlDecoderSetImageOutBuffer(jpegxlDecoder, &format, data, buffer_size);
-					if (JXL_DEC_SUCCESS != jpegxlStatus) {
+					if (unlikely(JXL_DEC_SUCCESS != jpegxlStatus)) {
 						vlog.error("JPEGXL: JxlDecoderSetImageOutBuffer() failed");
 						if (data != buf)
 							free(data);
@@ -524,24 +527,24 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 
 		memset(&welsDstBufInfo, 0, sizeof(SBufferInfo));
 		rv = WelsCreateDecoder(&welsSvcDecoderP);
-		if (rv != cmResultSuccess) {
+		if (unlikely(rv != cmResultSuccess)) {
 			vlog.error("WelsCreateDecoder() failed with error: %i", rv);
 			return;
 		}
 		rv = welsSvcDecoderP->Initialize(&welsParam);
-		if (rv != cmResultSuccess) {
+		if (unlikely(rv != cmResultSuccess)) {
 			vlog.error("welsSvcDecoderP->Initialize() failed with error: %i", rv);
 			WelsDestroyDecoder(welsSvcDecoderP);
 			return;
 		}
 		rv = welsSvcDecoderP->DecodeFrameNoDelay(srcBuf, srcBufLen, welsData, &welsDstBufInfo);
-		if (rv != cmResultSuccess) {
+		if (unlikely(rv != cmResultSuccess)) {
 			vlog.error("welsSvcDecoderP->DecodeFrameNoDelay() failed with error: %i", rv);
 			welsSvcDecoderP->Uninitialize();
 			WelsDestroyDecoder(welsSvcDecoderP);
 			return;
 		}
-		if (welsDstBufInfo.iBufferStatus <= 0) {
+		if (unlikely(welsDstBufInfo.iBufferStatus <= 0)) {
 			vlog.error("decoded H264 image without data");
 			welsSvcDecoderP->Uninitialize();
 			WelsDestroyDecoder(welsSvcDecoderP);
@@ -551,8 +554,9 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		mpHeight = welsDstBufInfo.UsrData.sSystemBuffer.iHeight;
 
 		/* sanity checks */
-		if (   (mpWidth != w && mpWidth != w + 1)
-		    || (mpHeight != h && mpHeight != h + 1)
+		if (unlikely(   (mpWidth != w && mpWidth != w + 1)
+			     || (mpHeight != h && mpHeight != h + 1)
+			    )
 		   ) {
 			vlog.debug("H264: rect size mismatch: expected %u-%u x %u-%u, got MP image with %ux%u", w, w + 1, h, h + 1, mpWidth, mpHeight);
 			welsSvcDecoderP->Uninitialize();
@@ -561,7 +565,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		}
 		mpPixelsize = 4;
 		mpStride = mpWidth * mpPixelsize;
-		if (is888) {
+		if (likely(is888)) {
 			/* direct */
 			vlog.verbose2("direct: compression: %u (%s), mpLevel: %u, x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpStride: %u, raw size: %u, compressed: %u", mpCompression, mpCompressionName(mpCompression), mpLevel, r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpStride, h * w * mpPixelsize, srcBufLen);
 			i420ToRgba(buf, stride * targetBufPixelsize, w, h, welsData[0], welsData[1], welsData[2], welsDstBufInfo.UsrData.sSystemBuffer.iStride[0], welsDstBufInfo.UsrData.sSystemBuffer.iStride[1]);
@@ -570,7 +574,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 			return;
 		}
 		data = (rdr::U8 *) malloc(mpHeight * mpStride);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("H264: failed to alloc memory for RGBA data");
 			welsSvcDecoderP->Uninitialize();
 			WelsDestroyDecoder(welsSvcDecoderP);
@@ -592,20 +596,20 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		 */
 		expectedDataSize = mpWidth * mpHeight * 3 / 2 + 2 * ((mpWidth & 1) * mpHeight + (mpHeight & 1) * mpWidth);
 //		vlog.verbose2("RAWI420: x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpWidth: %u, mpHeight: %u, mpStride: %u, raw size: %u, compressed: %u", r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpWidth, mpHeight, mpStride, h * w * mpPixelsize, srcBufLen);
-		if (srcBufLen != expectedDataSize) {
+		if (unlikely(srcBufLen != expectedDataSize)) {
 			vlog.error("RAWI420: buffer size mismatch: expected %u, got %u", expectedDataSize, srcBufLen);
 			return;
 		}
 		srcBufU = srcBuf + mpWidth * mpHeight;
 		srcBufV = srcBufU + mpWidth * mpHeight / 4 + (mpWidth & 1) * mpHeight + (mpHeight & 1) * mpWidth;
-		if (is888) {
+		if (likely(is888)) {
 			/* direct */
 			vlog.verbose2("direct: compression: %u (%s), mpLevel: %u, x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpStride: %u, raw size: %u, compressed: %u", mpCompression, mpCompressionName(mpCompression), mpLevel, r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpStride, h * w * mpPixelsize, srcBufLen);
 			i420ToRgba(buf, stride * targetBufPixelsize, mpWidth, mpHeight, srcBuf, srcBufU, srcBufV, mpWidth, (mpWidth + 1) / 2);
 			return;
 		}
 		data = (rdr::U8 *) malloc(mpHeight * mpStride);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("RAWI420: failed to alloc memory for RGBA data");
 			return;
 		}
@@ -619,17 +623,17 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		mpStride = mpWidth * mpPixelsize;
 //		vlog.verbose2("ZSTD: x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpWidth: %u, mpHeight: %u, mpStride: %u, raw size: %u, compressed: %u", r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpWidth, mpHeight, mpStride, h * w * mpPixelsize, srcBufLen);
 		dataSize = ZSTD_getFrameContentSize(srcBuf, srcBufLen);
-		if (dataSize != mpStride * mpHeight) {
+		if (unlikely(dataSize != mpStride * mpHeight)) {
 			vlog.error("ZSTD: data size mismatch: expected %u, got %i", mpStride * mpHeight, dataSize);
 			return;
 		}
 		data = (rdr::U8 *) malloc(dataSize);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("ZSTD: failed to alloc memory for RGBA data");
 			return;
 		}
 		dataSize = ZSTD_decompress(data, dataSize, srcBuf, srcBufLen);
-		if (dataSize < 0) {
+		if (unlikely(dataSize < 0)) {
 			vlog.error("ZSTD: ZSTD_decompress() failed with error %i", dataSize);
 			free(data);
 			return;
@@ -649,23 +653,23 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 //		vlog.verbose2("I420+ZSTD: x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpWidth: %u, mpHeight: %u, mpStride: %u, raw size: %u, compressed: %u", r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpWidth, mpHeight, mpStride, h * w * mpPixelsize, srcBufLen);
 		expectedUncompSize = mpWidth * mpHeight * 3 / 2 + 2 * ((mpWidth & 1) * mpHeight + (mpHeight & 1) * mpWidth);
 		uncompBufSize = ZSTD_getFrameContentSize(srcBuf, srcBufLen);
-		if (uncompBufSize != expectedUncompSize) {
+		if (unlikely(uncompBufSize != expectedUncompSize)) {
 			vlog.error("I420ZSTD: uncompressed data size mismatch: expected %u, got %i", expectedUncompSize, uncompBufSize);
 			return;
 		}
 		uncompBuf = (rdr::U8 *) malloc(uncompBufSize);
-		if (!uncompBuf) {
+		if (unlikely(!uncompBuf)) {
 			vlog.error("I420+ZSTD: failed to alloc memory for uncompressed data");
 			return;
 		}
 		uncompBufSize = ZSTD_decompress(uncompBuf, uncompBufSize, srcBuf, srcBufLen);
-		if (uncompBufSize < 0) {
+		if (unlikely(uncompBufSize < 0)) {
 			vlog.error("I420+ZSTD: ZSTD_decompress() failed with error %i", uncompBufSize);
 			return;
 		}
 		uncompBufU = uncompBuf + mpWidth * mpHeight;
 		uncompBufV = uncompBufU + mpWidth * mpHeight / 4 + (mpWidth & 1) * mpHeight + (mpHeight & 1) * mpWidth;
-		if (is888) {
+		if (likely(is888)) {
 			/* direct */
 			vlog.verbose2("direct: compression: %u (%s), mpLevel: %u, x: %u, y: %u, w: %u, h: %u, mpPixelsize: %u, stride: %u, mpStride: %u, raw size: %u, compressed: %u", mpCompression, mpCompressionName(mpCompression), mpLevel, r.tl.x, r.tl.y, w, h, mpPixelsize, stride, mpStride, h * w * mpPixelsize, srcBufLen);
 			i420ToRgba(buf, stride * targetBufPixelsize, mpWidth, mpHeight, uncompBuf, uncompBufU, uncompBufV, mpWidth, (mpWidth + 1) / 2);
@@ -673,7 +677,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 			return;
 		}
 		data = (rdr::U8 *) malloc(mpStride * mpHeight);
-		if (!data) {
+		if (unlikely(!data)) {
 			vlog.error("I420+ZSTD: failed to alloc memory for RGBA data");
 			free(uncompBuf);
 			return;
@@ -684,10 +688,10 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 		vlog.error("invalid MP compression: %u, mpLevel: %u,w %u, h %u", mpCompression, mpLevel, w, h);
 		return;
 	}
-	if (!is888) {
+	if (unlikely(!is888)) {
 		vlog.verbose("not is888, use transBuf: w: %u, h: %u, stride: %u, mpWidth: %u, mpStride: %u", w, h, stride, mpWidth, mpStride);
 		transBuf = (rdr::U8 *) malloc(w * h * targetBufPixelsize);
-		if (!transBuf) {
+		if (unlikely(!transBuf)) {
 			vlog.error("failed to alloc transBuf");
 			if (mpCompression == MP_COMPRESSION_RAWI420)
 				free(data);
@@ -763,7 +767,7 @@ void MPDecompressor::decompress(const rdr::U8 *mpBuf, int mpBufLen,
 	} else if (mpCompression == MP_COMPRESSION_I420ZSTD) {
 		free(data);
 	}
-	if (!is888) {
+	if (unlikely(!is888)) {
 		if (mpPixelsize == targetBufPixelsize) {
 			static const PixelFormat pfRGBX(32, 24, false, true, 255, 255, 255, 0, 8, 16);
 
