@@ -1,4 +1,4 @@
-/* Copyright 2017-2024 m-privacy GmbH.
+/* Copyright 2017-2025 m-privacy GmbH.
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,10 +16,13 @@
  * USA.
  */
 
+#if !defined(WIN32) && !defined(WIN64) && !defined(__APPLE__)
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <algorithm>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -43,6 +46,7 @@
 
 #include <mp_utils.h>
 #include <tgpro_environment.h>
+#include <transfer_log_helpers.h>
 
 #include "SAutotransferMime.h"
 
@@ -113,20 +117,6 @@ void SAutotransferMime::processFile(char* filename)
 }
 
 bool SAutotransferMime::checkFile(char* filename) {
-	// Check file size
-	struct stat buffer;
-	int status = stat(filename, &buffer);
-	if (status == 0) {
-		vlog.debug("%s: file size: %zu", __func__, buffer.st_size);
-		if (buffer.st_size == 0) {
-			vlog.debug("%s: Not processing file '%s' because of file size (0 bytes)", __func__, filename);
-			return false;
-		}
-	} else {
-		vlog.error("%s: error: %s. Couldn't execute stat on '%s'. Skipping...", __func__, strerror(errno), filename);
-		return false;
-	}
-
 	// Do not notify for .part files (firefox downloads)
 	if (ends_with(filename, ".part")) {
 		vlog.debug("%s: Not processing file '%s' because of its file ending (.part)", __func__, filename);
@@ -143,6 +133,22 @@ bool SAutotransferMime::checkFile(char* filename) {
 		return false;
 	}
 
+	// Check file size
+	vlog.verbose("%s: Sleeping 200 ms before processing file '%s' to avoid 0 bytes false alarms", __func__, filename);
+	usleep(200000); // Sleep 200 ms to try to avoid getting a timing problem that causes some false 0-byte-file positives.
+	struct stat buffer;
+	int status = stat(filename, &buffer);
+	if (status == 0) {
+		vlog.debug("%s: file size: %zu", __func__, buffer.st_size);
+		if (buffer.st_size == 0) {
+			vlog.debug("%s: Not processing file '%s' because of file size (0 bytes)", __func__, filename);
+			return false;
+		}
+	} else {
+		vlog.error("%s: error: %s. Couldn't execute stat on '%s'. Skipping...", __func__, strerror(errno), filename);
+		return false;
+	}
+
 	int checkMagicResult = checkMagic(filename);
 	if (checkMagicResult == MAGIC_NOT_ALLOWED) {
 		vlog.debug("%s: Not processing file '%s' because the MIME type doesn't allow it", __func__, filename);
@@ -158,12 +164,12 @@ bool SAutotransferMime::checkFile(char* filename) {
 
 void SAutotransferMime::checkGlibPacker(char * filename, const char * mimeTypeLibmagic)
 {
-	char glibtype[BUFFER_SIZE];
+//	char glibtype[BUFFER_SIZE];
 
-	if (get_mime_type_glib(filename, (char (*)[4096]) glibtype) && strcmp(glibtype, mimeTypeLibmagic)) {
-		vlog.info("%s (child (pid=%d)): warning: mime type: %s, file: \"%s\", glib shows different type %s!", __func__, getpid(), mimeTypeLibmagic, filename, glibtype);
-		syslog(LOG_DEBUG, "warning: mime type: %s, file: \"%s\", glib shows different type %s!", mimeTypeLibmagic, filename, glibtype);
-	}
+//	if (get_mime_type_glib(filename, (char (*)[4096]) glibtype) && strcmp(glibtype, mimeTypeLibmagic)) {
+//		vlog.info("%s (child (pid=%d)): warning: mime type: %s, file: \"%s\", glib shows different type %s!", __func__, getpid(), mimeTypeLibmagic, filename, glibtype);
+//		syslog(LOG_DEBUG, "warning: mime type: %s, file: \"%s\", glib shows different type %s!", mimeTypeLibmagic, filename, glibtype);
+//	}
 	for (const char* packerCheckType : packerCheckTypes) {
 		if (!strncasecmp(mimeTypeLibmagic, packerCheckType, strlen(packerCheckType))) {
 			char packercheckcmd[4096];
@@ -188,7 +194,56 @@ void SAutotransferMime::checkGlibPacker(char * filename, const char * mimeTypeLi
 	}
 }
 
-bool SAutotransferMime::checkSingleFile(char *filename, char **mimeTypeLibmagicP)
+static int containsMacros(const char* file)
+{
+	if (!file) {
+		vlog.error("%s: Invalid file pointer", __func__);
+		return 0;
+	}
+
+	size_t unzipCommandSize = snprintf(NULL, 0, "unzip -l \"%s\" > /dev/null 2>&1", file) + 1;
+	char* unzipCommand = (char*) malloc(unzipCommandSize);
+	if (!unzipCommand) {
+		vlog.error("%s: Memory allocation failed", __func__);
+		return 0;
+	}
+	snprintf(unzipCommand, unzipCommandSize, "unzip -l \"%s\" > /dev/null 2>&1", file);
+	unzipCommand[unzipCommandSize - 1] = '\0';
+
+	int status = system(unzipCommand);
+	free(unzipCommand);
+	if (status != 0) {
+		// If 'unzip -l' fails (non-zero exit code), assume it has macros
+		return 1;
+	}
+
+	size_t unzipAndEgrepCommandSize = snprintf(NULL, 0, "unzip -l \"%s\" | grep -v \"^Archive:\" | egrep -iq \"basic|scripts|macros|vb\"", file) + 1;
+	char* unzipAndEgrepCommand = (char*) malloc(unzipAndEgrepCommandSize);
+	if (!unzipAndEgrepCommand) {
+		vlog.error("%s: Memory allocation failed", __func__);
+		return 0;
+	}
+	snprintf(unzipAndEgrepCommand, unzipAndEgrepCommandSize, "unzip -l \"%s\" | grep -v \"^Archive:\" | egrep -iq \"basic|scripts|macros|vb\"", file);
+	unzipAndEgrepCommand[unzipAndEgrepCommandSize - 1] = '\0';
+
+	status = system(unzipAndEgrepCommand);
+	free(unzipAndEgrepCommand);
+
+	// If 'egrep' finds a match, it returns 0 (success), otherwise returns 1
+	return (WIFEXITED(status) && WEXITSTATUS(status) == 0) ? 1 : 0;
+}
+
+static bool listContains(const std::list<const char*>& allowedMimeTypes, const char* element)
+{
+	for (const char* allowedMimeType : allowedMimeTypes) {
+		if(!strcmp(element, allowedMimeType)) {
+			return true;
+		}
+    }
+	return false;
+}
+
+bool SAutotransferMime::checkSingleFile(char *filename, char **mimeTypeLibmagicP, char **reasonP)
 {
 	const char *magicName;
 	char *mmagic = NULL;
@@ -207,6 +262,23 @@ bool SAutotransferMime::checkSingleFile(char *filename, char **mimeTypeLibmagicP
 	else
 		mimeTypeLibmagic = mmagic;
 
+	// application/msword and application/vnc.ms- are actually older types which are not unzippable
+	// but since we now fail for unzippable files this is okay (and actually part of the requirements).
+	if (starts_with(mimeTypeLibmagic, "application/vnd.oasis.opendocument.") ||
+		starts_with(mimeTypeLibmagic, "application/vnd.openxmlformats-officedocument.")) {
+		if (allowedMimeTypes.empty() || listContains(allowedMimeTypes, "no-macros")) {
+			if (containsMacros(filename)) {
+				if (mimeTypeLibmagicP)
+					*mimeTypeLibmagicP = strdup(mimeTypeLibmagic);
+				if (reasonP)
+					*reasonP = strdup("no-macros");
+				vlog.info("%s (child (pid=%d)): rejected (no-macros): mime type: %s, file: \"%s\"", __func__, getpid(), mimeTypeLibmagic, filename);
+//				syslog(LOG_DEBUG, "rejected (no-macros): mime type: %s, file: \"%s\"", mimeTypeLibmagic, filename);
+				return false;
+			}
+		}
+	}
+
 	if (!strcmp(mimeTypeLibmagic, "application/octet-stream")) {
 		/* check for pkcs12 */
 		char pkcs12cmd[4096];
@@ -223,7 +295,8 @@ bool SAutotransferMime::checkSingleFile(char *filename, char **mimeTypeLibmagicP
 	if (allowedMimeTypes.empty()) {
 		if (mimeTypeLibmagicP)
 			vlog.debug("%s (child (pid=%d)): File \"%s\" detected MIME type '%s' is allowed", __func__, getpid(), filename, mimeTypeLibmagic);
-		checkGlibPacker(filename, mimeTypeLibmagic);
+		if(access("/etc/cu/sftpskiparchives", F_OK))
+			checkGlibPacker(filename, mimeTypeLibmagic);
 		return true;
 	}
 	for (const char* allowedMimeType : allowedMimeTypes) {
@@ -233,7 +306,8 @@ bool SAutotransferMime::checkSingleFile(char *filename, char **mimeTypeLibmagicP
 				vlog.debug("%s (child (pid=%d)): File \"%s\" detected MIME type '%s' is allowed", __func__, getpid(), filename, mimeTypeLibmagic);
 			else
 				vlog.verbose("%s (child (pid=%d)): File \"%s\" detected MIME type '%s' is allowed", __func__, getpid(), filename, mimeTypeLibmagic);
-			checkGlibPacker(filename, mimeTypeLibmagic);
+			if(access("/etc/cu/sftpskiparchives", F_OK))
+				checkGlibPacker(filename, mimeTypeLibmagic);
 			return true;
 		}
 	}
@@ -266,7 +340,7 @@ bool SAutotransferMime::checkTree(char *dirname)
 		if (entry->d_type == DT_DIR) {
 			result = checkTree(subpath);
 		} else if (entry->d_type == DT_REG) {
-			result = checkSingleFile(subpath, NULL);
+			result = checkSingleFile(subpath, NULL, NULL);
 		} else if (entry->d_type == DT_UNKNOWN) {
 			struct stat statbuf;
 
@@ -278,7 +352,7 @@ bool SAutotransferMime::checkTree(char *dirname)
 			if (S_ISDIR(statbuf.st_mode)) {
 				result = checkTree(subpath);
 			} else if (S_ISREG(statbuf.st_mode)) {
-				result = checkSingleFile(subpath, NULL);
+				result = checkSingleFile(subpath, NULL, NULL);
 			} else {
 				vlog.verbose("%s (child (pid=%d)): neither dir nor file for mime type: \"%s\"", __func__, getpid(), subpath);
 				continue;
@@ -303,6 +377,7 @@ bool SAutotransferMime::unpackAndCheck(char *name)
 	int err;
 	bool result;
 	struct stat statbuf;
+	pid_t pid;
 
 	if(lstat(name, &statbuf) < 0) {
 		vlog.info("%s (child (pid=%d)): failed to stat file \"%s\" for mime type, not unpacking: \"%s\"", __func__, getpid(), name, strerror(errno));
@@ -327,13 +402,20 @@ bool SAutotransferMime::unpackAndCheck(char *name)
 	if (err) {
 		vlog.info("%s (child (pid=%d)): failed to unpack \"%s\" for mime type", __func__, getpid(), name);
 		syslog(LOG_DEBUG, "failed to unpack \"%s\" for mime type", name);
-		snprintf(unpackcmd, 4095, "rm -rf \"%s\"", unpackdirname);
-		system(unpackcmd);
-		return false;
+		result = false;
+	} else {
+		result = checkTree(unpackdirname);
 	}
-	result = checkTree(unpackdirname);
-	snprintf(unpackcmd, 4095, "rm -rf \"%s\"", unpackdirname);
-	system(unpackcmd);
+	pid = fork();
+	if (pid <= 0) {
+		if (pid == 0)
+			daemon(0, 0);
+		snprintf(unpackcmd, 4095, "/usr/bin/rm -rf \"%s\"", unpackdirname);
+		system(unpackcmd);
+		if (pid == 0)
+			exit(0);
+	}
+	waitpid(-1, NULL, WNOHANG);
 	return result;
 }
 
@@ -352,6 +434,7 @@ int SAutotransferMime::checkMagic(char* filename) {
 	if (pid == 0){
 		bool magicOk;
 		char* mimeType;
+		char* reason = NULL;
 
 		// I am the child
 		vlog.verbose("%s (child (pid=%d)): Running in a separate process (fork)", __func__, getpid());
@@ -360,7 +443,7 @@ int SAutotransferMime::checkMagic(char* filename) {
 			_exit(2);
 		}
 
-		magicOk = checkSingleFile(filename, &mimeType);
+		magicOk = checkSingleFile(filename, &mimeType, &reason);
 
 		/* try to unpack ? */
 		if (!magicOk && mimeType) {
@@ -385,9 +468,11 @@ int SAutotransferMime::checkMagic(char* filename) {
 			}
 		}
 
-		logTransferIfNecessary(filename, mimeType ? mimeType : "unknown", magicOk);
+		logTransferIfNecessary(filename, mimeType ? mimeType : "unknown", reason, magicOk);
 		if (mimeType)
 			free(mimeType);
+		if (reason)
+			free(reason);
 		if (magicOk) {
 			vlog.verbose("%s (child (pid=%d)): '%s' allowed", __func__, getpid(), filename);
 			_exit(0);
@@ -442,10 +527,9 @@ int SAutotransferMime::checkMagic(char* filename) {
 	return MAGIC_FAILED;
 }
 
-void SAutotransferMime::logTransferIfNecessary(const char* filename, const char* mimeType, const bool magicOk) {
+void SAutotransferMime::logTransferIfNecessary(const char* filename, const char* mimeType, const char* reason, const bool magicOk) {
 	if (!magicOk || logAutotransfer) {
 		const unsigned int TRANSLOGLEN = 4096;
-		const char* username = getenv("USER");
 		const char* direction = "download"; // This is just here because I copy/pasted it from the sftp-server and prefer to keep the snprintf
 		const char* allowed = magicOk ? "allowed" : "rejected";
 
@@ -463,9 +547,27 @@ void SAutotransferMime::logTransferIfNecessary(const char* filename, const char*
 				vlog.error("%s: failed to calculate checksum for file: %s", __func__, filename);
 			}
 		}
+		char* new_username = resolve_username_for_logs();
+		if (!new_username) {
+			vlog.error("%s: Failed to resolve username for logs. Can't log to transfer.log", __func__);
+			return;
+		}
+		char* new_filename = replace_username_dir_for_logs(filename, new_username);
+		if (!new_filename) {
+			vlog.error("%s: Failed to replace username in dir for logs (for %s). Can't log to transfer.log", __func__, new_username);
+			free(new_username);
+			return;
+		}
 
 		char log[TRANSLOGLEN];
-		snprintf(log, TRANSLOGLEN, "\"%s\"%s mime type \"%s\", %s %s for user %s", filename, sha256_text, mimeType, allowed, direction, username);
+		if (reason)
+			snprintf(log, TRANSLOGLEN, "\"%s\"%s mime type \"%s\", %s %s for user %s, reason: %s", new_filename, sha256_text, mimeType, allowed, direction, new_username, reason);
+		else
+			snprintf(log, TRANSLOGLEN, "\"%s\"%s mime type \"%s\", %s %s for user %s", new_filename, sha256_text, mimeType, allowed, direction, new_username);
+
+		free(new_username);
+		free(new_filename);
+
 		log[TRANSLOGLEN - 1] = 0;
 		syslog(LOG_DEBUG, "%s", log);
 	}
@@ -535,3 +637,5 @@ void SAutotransferMime::initMagicFile() {
 	packerCheckTypes.push_back("video/");
 	packerCheckTypes.push_back("x-epoc/");
 }
+
+#endif /* #if !defined(WIN32) && !defined(WIN64) && !defined(__APPLE__) */
