@@ -58,6 +58,12 @@ rfb::IntParameter opswatTimeout("OpswatTimeout", "OPSWAP timeout in seconds (min
 char* rule = NULL;
 char* baseUrl = NULL;
 
+SAutotransferOpswat::SAutotransferOpswat() {
+	 snprintf(userAgent, 255, "TightGate-Pro OPSWAT client/%s (Build %s)", PACKAGE_VERSION, BUILD_TIMESTAMP);
+	 userAgent[255] = '\0';
+	 vlog.debug("%s: Set User-Agent to '%s' (for libcurl calls)", __func__, userAgent);
+}
+
 const char* SAutotransferOpswat::getBaseUrl()
 {
 	if (baseUrl == NULL) {
@@ -139,7 +145,7 @@ void SAutotransferOpswat::processFile(char* filename)
 	json uploadJsonResult = json::parse(uploadResult);
 	string dataId = uploadJsonResult["data_id"];
 
-	vlog.debug("%s: waiting max. %us for OPSWAT result for '%s' ...", __func__, (int) opswatTimeout, filename);
+	vlog.debug("%s: waiting max. %us for OPSWAT result for '%s' from '%s/file/%s'", __func__, (int) opswatTimeout, filename, getBaseUrl(), dataId.c_str());
 	int i = 0;
 	while (i < opswatTimeout) { // TODO: timeout? Aprox. 5 minutes if we sleep(1) in every loop (we don't really need high precision here)
 		string fileAnalyzeResult = getFileAnalyzeResultJson(dataId);
@@ -191,20 +197,30 @@ void SAutotransferOpswat::processFile(char* filename)
 					fileAnalyzeResultJson["process_info"].contains("post_processing") &&
 					fileAnalyzeResultJson["process_info"]["post_processing"].contains("converted_destination")) {
 
-				const char* tmpConvertedDestination = fileAnalyzeResultJson["process_info"]["post_processing"]["converted_destination"].get<std::string>().c_str();
-				if (tmpConvertedDestination && *tmpConvertedDestination) {
+				char* tmpConvertedDestination = strdup(fileAnalyzeResultJson["process_info"]["post_processing"]["converted_destination"].get<std::string>().c_str());
+				if (tmpConvertedDestination) {
+					vlog.debug("%s: found 'converted_destination' key in json analysis file: '%s'. Using it instead of the original filename", __func__, tmpConvertedDestination);
 					if (starts_with(tmpConvertedDestination, "(user: ")) {
-						const char* position = strstr(tmpConvertedDestination, ") "); // skip the (user: $USER) that we add ourselves
+						vlog.debug("%s: removing '(user: $USER)' from filename", __func__);
+						const char* position = strstr(tmpConvertedDestination, ") ");
 						if (position) {
 							size_t lengthToKeep = strlen(position + 2);
 							convertedDestination = (char*) malloc(lengthToKeep + 1);
-							strncpy(convertedDestination, position + 2, lengthToKeep);
-							convertedDestination[lengthToKeep] = '\0';
-							vlog.debug("%s: found 'converted_destination' key in json analysis file: '%s'. Using it instead of the original filename", __func__, convertedDestination);
+							if (convertedDestination) {
+								memcpy(convertedDestination, position + 2, lengthToKeep);  // Using memcpy instead of strncpy
+								convertedDestination[lengthToKeep] = '\0';
+								vlog.debug("%s: filename to save to after removing (user $USER): %s", __func__, convertedDestination);
+							} else {
+								vlog.error("%s: Failed to allocate memory", __func__);
+							}
 						}
+						free(tmpConvertedDestination);
 					} else { // If for whatever reason user: $USER isn't to be found anymore, just use the string as is
-						convertedDestination = strdup(tmpConvertedDestination);
+						convertedDestination = tmpConvertedDestination;
+						tmpConvertedDestination = NULL;  // Prevent double-free
 					}
+				} else {
+					vlog.error("%s: Failed to allocate memory (or to read 'converted_destination' from json although it supposedly is there)", __func__);
 				}
 			} else {
 				vlog.debug("%s: json analysis result doesn't contain a key [\"process_info\"][\"post_processing\"][\"converted_destination\"]. Using the original filename: %s", __func__, filename);
@@ -264,10 +280,10 @@ void SAutotransferOpswat::processFile(char* filename)
 							free(line);
 						}
 					} else {
-						vlog.info("%s: failed to open logfile %s, not logging, error: %s", __func__, logpath, strerror(errno));
+						vlog.error("%s: failed to open logfile %s, not logging, error: %s (notifications with mp-watch won't work!)", __func__, logpath, strerror(errno));
 					}
 				} else {
-					vlog.debug("%s: logfile %s not found, not logging", __func__, logpath);
+					vlog.error("%s: logfile %s not found, not logging (notifications with mp-watch won't work!)", __func__, logpath);
 				}
 				return;
 			} else {
@@ -368,6 +384,7 @@ string SAutotransferOpswat::getFileAnalyzeResultJson(const string dataId)
 		if (file_exists(CA_CERTS_FILE)) {
 			curl_easy_setopt(curl, CURLOPT_CAINFO, CA_CERTS_FILE);
 		}
+		curl_easy_setopt(curl, CURLOPT_USERAGENT, userAgent);
 		curl_easy_setopt(curl, CURLOPT_URL, url);
 		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writeCallback);
 		curl_easy_setopt(curl, CURLOPT_WRITEDATA, &json);
@@ -413,6 +430,7 @@ CURLcode SAutotransferOpswat::getSanitizedFile(const string dataId, const string
 			curl_easy_setopt(curl, CURLOPT_CAINFO, CA_CERTS_FILE);
 		}
 		fp = fopen(outputFile.c_str(), "wb");
+		curl_easy_setopt(curl, CURLOPT_USERAGENT, userAgent);
 		curl_easy_setopt(curl, CURLOPT_URL, url);
 		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, NULL); // We don't need to write a callback function because it's the "standard" way
 		curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
@@ -495,6 +513,7 @@ string SAutotransferOpswat::uploadFile(const char* filename)
 			curl_easy_setopt(curl, CURLOPT_CAINFO, CA_CERTS_FILE);
 		}
 		curl_easy_setopt(curl, CURLOPT_URL, url);
+		curl_easy_setopt(curl, CURLOPT_USERAGENT, userAgent);
 		curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
 		curl_easy_setopt(curl, CURLOPT_POST, 1L);
 		curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, getFilesize(filename));
