@@ -166,7 +166,18 @@ char metadata_json[4096];
 // Function declaration for removeNonPrintableCharacters
 void removeNonPrintableCharacters(char* str, size_t size);
 
-bool createJob(char* jobIdOutput, char* tokenOutput) {
+void checkCACertificate(const char* caCertPath) {
+	if (access(caCertPath, F_OK) != -1) {
+		// File exists
+		vlog.debug("%s: CA certificate file found at %s", __func__, caCertPath);
+	} else {
+		// File does not exist, log error and exit
+		vlog.error("%s: CA certificate file not found at %s", __func__, caCertPath);
+		exit(EXIT_FAILURE); // Exit with an error code
+	}
+}
+
+bool createJob(char* jobIdOutput, char* tokenOutput, const char* caCertPath) {
 	// Define the job creation JSON
 	const char* scanners = getVaithexScannersAsJsonStr();
 	if (strlen(scanners) == 0) {
@@ -176,23 +187,28 @@ bool createJob(char* jobIdOutput, char* tokenOutput) {
 		// Scanners selected, include scanners_enabled in the JSON
 		snprintf(jobsPostBody, sizeof(jobsPostBody), "{\"normalize\": true,\"scanners_enabled\": %s,\"server_timeout_sec\":0}", scanners);
 	}
+	// Check if the CA certificate file exists
+	checkCACertificate(caCertPath);
 
-	// Build the command to create the job
+	// Build the wget command with the CA certificate
 	char getJobCmd[1500];
-	snprintf(getJobCmd, sizeof(getJobCmd), "wget --no-check-certificate --quiet --progress=dot --method=POST --output-document=- --body-data='%s' 'https://%s:%s/v1/jobs'", jobsPostBody, vaithexIp.getData(), vaithexPort.getData());
-	vlog.debug("%s: running command: %s", __func__, getJobCmd);
+	snprintf(getJobCmd, sizeof(getJobCmd),"wget --ca-certificate=%s --quiet --progress=dot --method=POST --output-document=- --body-data='%s' 'https://%s:%s/v1/jobs'", caCertPath, jobsPostBody, vaithexIp.getData(), vaithexPort.getData());
 
+	vlog.debug("%s: running command: %s", __func__, getJobCmd);
 	FILE* responseStream = popen(getJobCmd, "r");
 	if (!responseStream) {
 		vlog.error("%s: Failed to execute command", __func__);
 		return false;
 	}
-
 	// Read the response into the buffer
 	char response_buffer[4096]; // Adjust the buffer size as needed
 	size_t responseSize = fread(response_buffer, 1, sizeof(response_buffer) - 1, responseStream);
+	if (responseSize == (size_t)-1) {
+		vlog.error("%s: Failed to read response from the stream", __func__);
+		pclose(responseStream);
+		return false;
+	}
 	pclose(responseStream);
-
 	if (responseSize == 0) {
 		vlog.error("%s: Empty response from the server", __func__);
 		return false;
@@ -211,10 +227,10 @@ bool createJob(char* jobIdOutput, char* tokenOutput) {
 		if (response_json.contains("id") && response_json.contains("token")) {
 			job_id = response_json["id"];
 			token = response_json["token"];
-			//Store the job ID and token (in output parameters)
+			// Store the job ID and token (in output parameters)
 			snprintf(jobIdOutput, 4096, "%ld", job_id);
 			snprintf(tokenOutput, 4096, "%s", token.c_str());
-			//Log job ID and token
+			// Log job ID and token
 			vlog.debug("%s: Job ID: %ld, Token: %s", __func__, job_id, token.c_str());
 		} else {
 			vlog.error("%s: JSON response is missing 'id' or 'token' fields", __func__);
@@ -240,8 +256,7 @@ void removeNonPrintableCharacters(char* str, size_t size) {
 	str[j] = '\0';
 }
 
-void uploadPayload(char* jobIdOutput, char* tokenOutput, const char* filename, const char* dir, const char* uploadUrlParam)
-{
+void uploadPayload(char* jobIdOutput, char* tokenOutput, const char* filename, const char* dir, const char* caCertPath) {
 	// Print request for the upload payload
 	vlog.debug("%s: Uploading payload for filename: %s", __func__, filename);
 
@@ -253,18 +268,14 @@ void uploadPayload(char* jobIdOutput, char* tokenOutput, const char* filename, c
 	char tar_cmd[700];
 	snprintf(tar_cmd, sizeof(tar_cmd), "tar -cf '/tmp/%s.tar' -C '%s' '%s'", filename, dir, filename);
 	system(tar_cmd);
-	// Log the tar command
 	vlog.info("%s: Executing tar cmd: %s", __func__, tar_cmd);
-	// Build the cmd to upload the payload using wget
+
+	// Construct the upload command with the CA certificate
 	char uploadCmd[1500];
+	snprintf(uploadCmd, sizeof(uploadCmd), "wget --ca-certificate=%s --quiet --progress=dot --method=POST --output-document=- --body-file='/tmp/%s.tar' --header='X-Waas-Token: %s' '%s'", caCertPath, filename, tokenOutput, uploadUrl);
 
-	// Construct the upload command
-	snprintf(uploadCmd, sizeof(uploadCmd), "wget --no-check-certificate --quiet --progress=dot --method=POST --output-document=- --body-file='/tmp/%s.tar' --header='X-Waas-Token: %s' '%s'", filename, tokenOutput, uploadUrl);
-
-	// Print request for the uploadCmd
+	// Print and execute the wget upload command
 	vlog.debug("%s: Executing wget cmd for uploadPayload: %s", __func__, uploadCmd);
-
-	// Execute the cmd to upload the payload
 	int uploadResult = system(uploadCmd);
 	if (uploadResult == 0) {
 		vlog.debug("%s: Payload uploaded for filename: %s", __func__, filename);
@@ -272,10 +283,8 @@ void uploadPayload(char* jobIdOutput, char* tokenOutput, const char* filename, c
 		vlog.debug("%s: Payload upload failed for filename: %s. Error code: %d", __func__, filename, uploadResult);
 		return;
 	}
-
 	// Check the name of the uploaded tar file
 	vlog.debug("Uploaded tar file name: %s.tar", filename);
-
 	// Clean up the temporary tarball file
 	char cleanupCmd[1000];
 	snprintf(cleanupCmd, sizeof(cleanupCmd), "rm -f '/tmp/%s.tar'", filename);
@@ -286,18 +295,17 @@ void uploadPayload(char* jobIdOutput, char* tokenOutput, const char* filename, c
 	free(tokenOutput);
 }
 
-void fetchMetadata(const char* token, long job_id, char* metadata_json_buffer) {
+void fetchMetadata(const char* token, long job_id, char* metadata_json_buffer, const char* caCertPath) {
 	// Build the URL to fetch metadata
 	char metadata_url[600];
 	snprintf(metadata_url, sizeof(metadata_url), "https://%s:%s/v1/jobs/%ld", vaithexIp.getData(), vaithexPort.getData(), job_id);
-
-	// Build the command to fetch metadata using wget
+	// Build the command to fetch metadata using wget with the CA certificate
 	char metadata_cmd[1000];
-	snprintf(metadata_cmd, sizeof(metadata_cmd), "wget --no-check-certificate --quiet --method=GET --output-document=- --header='X-Waas-Token: %s' '%s'", token, metadata_url);
+	snprintf(metadata_cmd, sizeof(metadata_cmd), "wget --ca-certificate=%s --quiet --method=GET --output-document=- --header='X-Waas-Token: %s' '%s'", caCertPath, token, metadata_url);
 	// Log the metadata_cmd
 	vlog.debug("%s: Executing the wget metadata cmd: %s", __func__, metadata_cmd);
 
-// 	// Execute the command to fetch the metadata
+	// Execute the command to fetch the metadata
 	FILE* metadata_output = popen(metadata_cmd, "r");
 	if (metadata_output == NULL) {
 		// Log the error
@@ -315,10 +323,8 @@ void fetchMetadata(const char* token, long job_id, char* metadata_json_buffer) {
 	pclose(metadata_output);
 	// Log the complete response
 	vlog.debug("%s: Complete metadata response: %s", __func__, metadata_response);
-
 	// Remove non-printable characters from the metadata_response
 	removeNonPrintableCharacters(metadata_response, sizeof(metadata_response));
-
 	// Parse the metadata response using nlohmann/json
 	json parsed_metadata_json;
 	try {
@@ -327,8 +333,7 @@ void fetchMetadata(const char* token, long job_id, char* metadata_json_buffer) {
 		vlog.error("JSON parsing error: %s", e.what());
 		return;
 	}
-	/*Assuming metadata_json_buffer is a character buffer passed to the function and
-	 you want to store the parsed JSON in that buffer */
+	// Assuming metadata_json_buffer is a character buffer passed to the function and you want to store the parsed JSON in that buffer
 	snprintf(metadata_json_buffer, 4096, "%s", parsed_metadata_json.dump().c_str());
 }
 
@@ -338,7 +343,7 @@ const char* outputDir = "/tmp/vaithex-tmp";
 // Function prototype for extractAll
 static int extractAll(const char* filename, const char* outputDir, char* lastExtractedFile);
 
-void processCleanOrNormalized(const char* filePath, const char* filename, const char* metadata_response) {
+void processCleanOrNormalized(const char* filePath, const char* filename, const char* metadata_response, const char* caCertPath) {
 	json metadata_json;
 	try {
 		metadata_json = json::parse(metadata_response);
@@ -391,7 +396,7 @@ void processCleanOrNormalized(const char* filePath, const char* filename, const
 
 				// Execute the wget command to fetch the result payload
 				char result_payload_cmd[4096];
-				snprintf(result_payload_cmd, sizeof(result_payload_cmd), "wget --no-check-certificate --quiet --progress=dot --method=GET --output-document='%s' --header='X-Waas-Token: %s' '%s'", tarredResult, token.c_str(), result_payload_url);
+				snprintf(result_payload_cmd, sizeof(result_payload_cmd), "wget --ca-certificate=%s --quiet --progress=dot --method=GET --output-document='%s' --header='X-Waas-Token: %s' '%s'",caCertPath, tarredResult, token.c_str(), result_payload_url);
 				vlog.debug("Running: %s", result_payload_cmd);
 
 				if (system(result_payload_cmd) == 0) {
@@ -442,10 +447,10 @@ void processCleanOrNormalized(const char* filePath, const char* filename, const
 					}
 					vlog.debug("Filename %s has been modified to: %s", normalizedFilename, modifiedFilename.c_str());
 
-// 					// Append "_vaithexed" to the normalized filename
-// 					std::string modifiedFilename = std::string(normalizedFilename) + "___vaithexed";
-// 					//Log the file name and the modified file name
-// 					vlog.debug("Filename: %s has been modified to: %s", normalizedFilename, modifiedFilename.c_str());
+ 					//// Append "_vaithexed" to the normalized filename
+ 					//std::string modifiedFilename = std::string(normalizedFilename) + "___vaithexed";
+					////Log the file name and the modified file name
+					//vlog.debug("Filename: %s has been modified to: %s", normalizedFilename, modifiedFilename.c_str());
 
 					std::string autotransferNowFile = "/home/user/.autotransfer-now/" + std::string(getpwuid(geteuid())->pw_name) + "/" + modifiedFilename;
 					vlog.debug("%s: trying to mp_move '%s' to '%s'", __func__, extractedFile, autotransferNowFile.c_str());
@@ -512,7 +517,6 @@ void processInfected(const char* filePath, const char* filename, const char* met
 						struct tm * now = localtime(&nowsec);
 						snprintf(line, 4095, "%s %02u.%02u.%u %02u:%02u:%02u %s %s\n", hostname, now->tm_mday, now->tm_mon+1, now->tm_year+1900, now->tm_hour, now->tm_min, now->tm_sec, filename, filePath);
 
-
 						vlog.debug("%s: logging '%s' to %s", __func__, line, logpath);
 						fwrite(line, 1, strlen(line), handle);
 						if (ferror(handle)) {
@@ -529,10 +533,10 @@ void processInfected(const char* filePath, const char* filename, const char* met
 						free(line);
 					}
 				} else {
-					vlog.info("%s: failed to open logfile %s, not logging, error: %s", __func__, logpath, strerror(errno));
+					vlog.error("%s: failed to open logfile %s, not logging, error: %s (notifications with mp-watch won't work!)", __func__, logpath, strerror(errno));
 				}
 			} else {
-				vlog.debug("%s: logfile %s not found, not logging", __func__, logpath);
+				vlog.error("%s: logfile %s not found, not logging (notifications with mp-watch won't work!)", __func__, logpath);
 			}
 
 			return;
@@ -595,15 +599,21 @@ void SAutotransferVaithex::processFile(char *filePath)
 	char* jobIdOutput = (char*) malloc(4096);
 	char* tokenOutput = (char*) malloc(4096);
 
-	bool jobCreationSucceeded = createJob(jobIdOutput, tokenOutput);
+	// Define the CA certificate path
+	const char* caCertPath = "/home/user/.customvaithexca/ca-certificates";
+
+	// Pass the CA certificate path to createJob
+	bool jobCreationSucceeded = createJob(jobIdOutput, tokenOutput, caCertPath);
 	if (!jobCreationSucceeded) {
 		vlog.error("%s: failed to create a job. Aborting VAITHEXing of file '%s'", __func__, filename);
+		free(jobIdOutput); // Free memory to prevent leaks
+		free(tokenOutput); // Free memory to prevent leaks
 		return;
 	}
 	vlog.debug("Yay! Got me some id (%s) and some token (%s)", jobIdOutput, tokenOutput);
 
-	//Call uploadPayload function
-	uploadPayload(jobIdOutput, tokenOutput, filename, dir, uploadUrl);
+	// Call uploadPayload function with the CA certificate path
+	uploadPayload(jobIdOutput, tokenOutput, filename, dir, caCertPath);
 
 	// Create a buffer to store the response
 	char response_buffer[4096]; // Adjust the buffer size as needed;
@@ -616,10 +626,10 @@ void SAutotransferVaithex::processFile(char *filePath)
 
 	// Call the fetchMetadata function
 	char metadataOutput[4096];
-	fetchMetadata(tokenOutput, job_id, metadataOutput);
+	fetchMetadata(tokenOutput, job_id, metadataOutput, caCertPath);
 
 	// Call the function to process clean or normalized result
-	processCleanOrNormalized(filePath, filename, metadataOutput);
+	processCleanOrNormalized(filePath, filename, metadataOutput, caCertPath);
 
 	char*  autotransferNowFile = strdup(filePath);
 	autotransferNowFile[4095] = '\0';
