@@ -226,7 +226,6 @@ static int callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 				if (*usergroups) {
 					portable_setenv("TGGROUPS", usergroups);
 				} else {
-					mplog_info("juaaaaa from SSecurityKrb 1: setting TGGROUPS to empty");
 					portable_setenv("TGGROUPS", "empty");
 				}
 				rc = GSASL_OK;
@@ -242,28 +241,6 @@ static int callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 			break;
 		}
 
-		// char * ldap_server_1 = SSecurityKrb::KrbLdapBind.getData();
-		vlog.debug("Forcing ldap_server_1 to 'auto'");
-		char * ldap_server_1 = (char*) "auto";
-		// char * ldap_server_2 = SSecurityKrb::KrbLdapBind2.getData();
-		vlog.debug("Forcing ldap_server_2 to ''");
-		char * ldap_server_2 = (char*) "";
-		if ((!ldap_server_1 || !ldap_server_1[0]) && (!ldap_server_2 || !ldap_server_2[0])) {
-			vlog.debug("No LDAP server bind URI specified.");
-			rc = GSASL_NO_AUTHZID;
-			break;
-		}
-
-		// char * ldap_base = SSecurityKrb::KrbLdapBase.getData();
-		vlog.debug("Forcing LDAP base to 'auto'"); // See #0005665
-		char * ldap_base = (char*) malloc(4096); // To make sure we have enough space for the actual ldap_base (which is stored here too for some reason)
-		snprintf(ldap_base, 5, "auto");
-		ldap_base[4] = 0;
-		if (!ldap_base || !ldap_base[0]) {
-			vlog.debug("No LDAP base specified.");
-			break;
-		}
-
 		constccache = gsasl_property_fast(sctx, GSASL_GSSAPI_CCACHE);
 		if (!constccache || !constccache[0]) {
 			vlog.debug("No CCache provided (delegation not working): won't use client's ticket. Running kinit with keytab...");
@@ -280,68 +257,32 @@ static int callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 
 		char* domain = get_domain(realm, SSecurityKrb::KrbDomainFile);
 
-		// const char* ldap_auto = SSecurityKrb::KrbLdapAuto;
-		vlog.debug("Forcing ldap_auto to 'realm'");
-		const char* ldap_auto = "realm"; // It's irrelevant whether this is 'auto' or 'realm': there's only a difference between these two while "discovering" AD servers
-
-		// Check that if ldap_base is auto or that if either ldap_server_1 or 2 are set to auto,
-		// realm and domain are also set, and ldap_auto is set to 'yes', 'no' or 'auto'
-		rc = check_ldap_config_values(realm, domain, ldap_base, ldap_server_1, ldap_server_2, ldap_auto);
-		if (rc != MP_LDAP_OK) {
+		char* ldap_base = NULL; // generate_ldap_base mallocs mem!
+		rc = generate_ldap_base(&ldap_base, domain);
+		if (rc != MP_LDAP_OK || !ldap_base) {
+			vlog.error("Failed to generate ldap base");
+			if (ldap_base) {
+				free(ldap_base);
+			}
 			rc = GSASL_NO_AUTHZID;
 			break;
 		}
 
-		if (!strcmp(ldap_base, "auto")) {
+		vlog.debug("Generated ldap base: %s", ldap_base);
+
+		rc = perform_ldap_searches(domain, NULL, ldap_base, login_name, NULL);
+
+		vlog.verbose("perform_ldap_searches returned: rc (%d)", rc);
+		if (rc == MP_LDAP_OK) {
+			if (SSecurityKrb::KrbProvideCcache) {
+				if (constccache && constccache[0]) {
+					portable_setenv("KRBCCACHE", constccache);
+				} else {
+					vlog.error("KrbProvideCcache option is set to true, but constccache is null or empty, so we couldn't set the KRBCCACHE environment variable. Possibly delegation isn't allowed for tgpro's computer account. This is necessary in order to ask for authorization in the name of the user for a specific service (e.g. for an Active Directory proxy authentication).");
+				}
+			}
 			free(ldap_base);
-			ldap_base = NULL;
-			rc = generate_ldap_base(&ldap_base, domain);
-			if (!ldap_base) {
-				vlog.error("Failed to generate ldap base");
-				rc = GSASL_NO_AUTHZID;
-				break;
-			}
-			vlog.debug("Generated ldap base: %s", ldap_base);
-			if (rc != MP_LDAP_OK) {
-				rc = GSASL_NO_AUTHZID;
-				break;
-			}
-		}
-
-		if (ldap_server_1 && ldap_server_1[0]) {
-			// searches in plural because if ldap_server_1 is 'auto', a list of discovered ldap servers will be used
-			rc = perform_ldap_searches(domain, ldap_server_1, ldap_base, login_name, ldap_auto);
-
-			vlog.verbose("perform_ldap_searches returned: rc (%d)", rc);
-			if (rc == MP_LDAP_OK) {
-				if (SSecurityKrb::KrbProvideCcache) {
-					if (constccache && constccache[0]) {
-						portable_setenv("KRBCCACHE", constccache);
-					} else {
-						vlog.error("KrbProvideCcache option is set to true, but constccache is null or empty, so we couldn't set the KRBCCACHE environment variable. Possibly delegation isn't allowed for tgpro's computer account. This is necessary in order to ask for authorization in the name of the user for a specific service (e.g. for an Active Directory proxy authentication).");
-					}
-				}
-				return GSASL_OK; // If not, continue with ldap_server_2
-			}
-		}
-
-
-		if (ldap_server_2 && ldap_server_2[0]) {
-			// searches in plural because if ldap_server_1 is 'auto', a list of discovered ldap servers will be used
-			rc = perform_ldap_searches(domain, ldap_server_2, ldap_base, login_name, ldap_auto);
-
-			vlog.debug("perform_ldap_searches returned: rc (%d)", rc);
-			if (rc == MP_LDAP_OK) {
-				if (SSecurityKrb::KrbProvideCcache) {
-					if (constccache && constccache[0]) {
-						portable_setenv("KRBCCACHE", constccache);
-					} else {
-						vlog.error("KrbProvideCcache option is set to true, but constccache is null or empty, so we couldn't set the KRBCCACHE environment variable. Possibly delegation isn't allowed for tgpro's computer account. This is necessary in order to ask for authorization in the name of the user for a specific service (e.g. for an Active Directory proxy authentication).");
-					}
-				}
-
-				return GSASL_OK;
-			}
+			return GSASL_OK; // If not, continue with ldap_server_2
 		}
 
 		portable_unsetenv("KRB5CCNAME");
@@ -421,8 +362,10 @@ static int callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 				throw AuthFailureException("Could not get hostname!");
 			krbhostname[255] = 0;
 		}
-		gsasl_property_set(sctx, GSASL_HOSTNAME, krbhostname);
-		vlog.debug("set HOSTNAME to %s", krbhostname);
+		//		gsasl_property_set(sctx, GSASL_HOSTNAME, krbhostname);
+		//		vlog.debug("set HOSTNAME to %s", krbhostname);
+		gsasl_property_set(sctx, GSASL_HOSTNAME, "not-used-hostname.invalid");
+		vlog.debug("set HOSTNAME to %s (this is on purpose - it is just an easily revertible test to see whether we actually need the krbhostname on the server side, which I believe we might not)", "not-used-hostname.invalid");
 		rc = GSASL_OK;
 		break;
         }
