@@ -2,7 +2,7 @@
  * Copyright (C) 2004 Red Hat Inc.
  * Copyright (C) 2005 Martin Koegler
  * Copyright (C) 2010 TigerVNC Team
- * Copyright (C) 2015-2024 m-privacy GmbH
+ * Copyright (C) 2015-2025 m-privacy GmbH
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -88,7 +88,7 @@ static LogWriter vlog("SSecurityMulti");
 #define MAXPABUF 10 * 1024 * 1024
 
 IntParameter paReadBufferSize("PaReadBufferSize", "Buffer size for reads from Pulseaudio socket (min. 15KB, sanity limit at 10MB)", DEFPABUF, MINPABUF, MAXPABUF);
-StringParameter logoutMagicApp("LogoutMagicApp", "App to start for logout", "/usr/bin/lxde-logout");
+StringParameter logoutMagicApp("LogoutMagicApp", "App to start for logout", "/usr/local/bin/mplogout");
 StringParameter urlMagicApp("UrlMagicApp", "App to start on URL", "/usr/local/bin/firefoxauto");
 IntParameter keepAliveTimeout("KeepAliveTimeout", "Keep-alive timeout in seconds (0 to disable)", 150, 0);
 IntParameter initialNoAudio("InitialNoAudio", "time to block audio at startup in seconds (0 to disable)", 0, 0);
