@@ -47,7 +47,12 @@ class SWebcamHandler {
 		static threadMap_t ffmpegThreadId;
 
 		static THREAD_FUNC ffmpegThread(void* param);
+		static THREAD_FUNC fifoReaderThread(void* param);
 		static THREAD_FUNC udpSocketThread(void* param);
+		static THREAD_FUNC fallbackStillFrameLoopThread(void* param);
+		static THREAD_ID fifoReaderThreadId;
+		static int fifoReaderPid;
+		static THREAD_ID fallbackStillFrameLoopThreadId;
 };
 
 #endif /* __SWEBCAM_HANDLER_H__ */
