@@ -1,6 +1,6 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
  * Copyright 2012-2016 Pierre Ossman for Cendio AB
- * Copyright (C) 2015-2017 m-privacy GmbH
+ * Copyright (C) 2015-2025 m-privacy GmbH
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ StringParameter urlMagicApp("urlmagicapp", "App to start on URL", "/usr/bin/icew
 
 BoolParameter logoutMagic("logoutmagic", "Check clipboard for logout request", 0);
 StringParameter logoutMagicPrefix("logoutmagicstring", "Logout Magic String", "mplogout:");
-StringParameter logoutMagicApp("logoutmagicapp", "App to start for logout", "/usr/bin/lxde-logout");
+StringParameter logoutMagicApp("logoutmagicapp", "App to start for logout", "/usr/local/bin/mplogout");
 
 #define ACCEPT_CUT_TEXT "AcceptCutText"
 #define SEND_CUT_TEXT "SendCutText"
