@@ -1,7 +1,7 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
  * Copyright (C) 2011 D. R. Commander.  All Rights Reserved.
  * Copyright 2009-2014 Pierre Ossman for Cendio AB
- * Copyright (C) 2015-2021 m-privacy GmbH
+ * Copyright (C) 2015-2025 m-privacy GmbH
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -104,7 +104,7 @@ CConn::CConn(const char* vncServerName, network::Socket* socket=NULL)
   setShared(::shared);
   sock = socket;
 
-  supportsLocalCursor = true;
+  supportsLocalCursor = allowLocalCursor;
   supportsCursorPosition = true;
   supportsDesktopResize = true;
   supportsLEDState = false;
@@ -172,6 +172,8 @@ CConn::~CConn()
   if (desktop)
     delete desktop;
 
+  sock->shutdown();
+
   delete [] serverHost;
   if (sock)
     Fl::remove_fd(sock->getFd());
