@@ -173,15 +173,18 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
   }
   if (wa_w < w) {
     vlog.info("Too much width");
+#if 0
     w = wa_w;
     reduced = 1;
     geom_x = 0;
+#endif
   }
 
   // Add screen offset again which we had removed to do all the (relative) calculations
   geom_x += wa_x;
   geom_y += wa_y;
 
+#if 0
 #if (defined(WIN32))
   if ((wa_h - 36) < h) {
     vlog.info("Too much height");
@@ -194,6 +197,7 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
     reduced = 1;
     geom_y = 0;
   }
+#endif
   if (reduced || overlapping) {
     vlog.info("Reduced or overlapping!");
     vlog.info("Final re-calculated width: %d", w);
@@ -216,7 +220,7 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
 #ifdef __APPLE__
   // On OS X we can do the maximize thing properly before the
   // window is showned. Other platforms handled further down...
-  if (maximize) {
+  if (::maximize) {
     int dummy;
     Fl::screen_work_area(dummy, dummy, w, h, geom_x, geom_y);
   }
@@ -250,7 +254,7 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
   // maximized property on Windows and X11 before showing the window.
   // See STR #2083 and STR #2178
 #ifndef __APPLE__
-  if (maximize) {
+  if (::maximize) {
     maximizeWindow();
   }
 #endif
@@ -258,14 +262,6 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
   // Adjust layout now that we're visible and know our final size
   repositionWidgets();
 
-  if (delayedFullscreen) {
-    // Hack: Fullscreen requests may be ignored, so we need a timeout for
-    // when we should stop waiting. We also really need to wait for the
-    // resize, which can come after the fullscreen event.
-    Fl::add_timeout(0.5, handleFullscreenTimeout, this);
-    fullscreen_on();
-  }
-
   // Throughput graph for debugging
   if (vlog.getLevel() >= LogWriter::LEVEL_STATS) {
     memset(&stats, 0, sizeof(stats));
@@ -388,6 +384,7 @@ void DesktopWindow::resizeFramebuffer(int new_w, int new_h)
 
   XFree(atoms);
 #endif
+  maximize.setParam(maximized);
 
   // If we're letting the viewport match the window perfectly, then
   // keep things that way for the new size, otherwise just keep things
@@ -849,6 +846,14 @@ int DesktopWindow::handle(int event)
     else
       ungrabKeyboard();
 
+    // The window manager respected our full screen request, so stop
+    // waiting and delaying the session resize
+    if (delayedFullscreen && fullscreen_active()) {
+      Fl::remove_timeout(handleFullscreenTimeout, this);
+      delayedFullscreen = false;
+      remoteResize(w(), h());
+    }
+
     break;
 
   case FL_ENTER:
@@ -918,6 +923,19 @@ int DesktopWindow::fltkHandle(int event, Fl_Window *win)
       }
       break;
 
+    case FL_SHOW:
+      // In this particular place, FL_SHOW means an actual MapNotify,
+      // which means we can continue enabling initial fullscreen.
+      if (dw->delayedFullscreen) {
+        // Hack: Fullscreen requests may be ignored, so we need a
+        // timeout for when we should stop waiting. We also really need
+        // to wait for the resize, which can come after the fullscreen
+        // event.
+        Fl::add_timeout(0.5, handleFullscreenTimeout, dw);
+        dw->fullscreen_on();
+      }
+      break;
+
     case FL_RELEASE:
       // We usually fail to grab the mouse if a mouse button was
       // pressed when we gained focus (e.g. clicking on our window),
