@@ -24,6 +24,8 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <cerrno>
+
 #include <rfb/LogWriter.h>
 
 #include <FL/Fl.H>
@@ -42,6 +44,11 @@
 #include "win_registry_utils.h"
 #include <winsock2.h>
 #include <windows.h>
+#else
+#include <dirent.h>
+#include <regex>
+#include <sys/socket.h>
+#include <sys/un.h>
 #endif /*defined(WIN32) || defined(WIN64)*/
 
 #include "MagicUrlHelper.h"
@@ -445,6 +452,80 @@ void MagicUrlHelper::updateKdeglobals(char *path) {
 }
 #endif /*!defined(WIN32) && !defined(WIN64)*/
 
+#if !defined(WIN32) && !defined(WIN64) && !defined(__APPLE__)
+char* findOpenMagicurlSocket() {
+	const char* tmpDir = "/tmp";
+	uid_t currentUid = getuid();
+	pid_t currentPid = getpid();
+
+	vlog.debug("%s: Scanning directory '%s' for magicurl sockets", __func__, tmpDir);
+
+	DIR* dir = opendir(tmpDir);
+	if (!dir) {
+		vlog.error("%s: Failed to open directory '%s': %s", __func__, tmpDir, strerror(errno));
+		return NULL;
+	}
+
+	struct dirent* entry;
+	std::regex magicurlRegex("^\\.magicurl([0-9]+)$");
+
+	while ((entry = readdir(dir)) != NULL) {
+		vlog.debug("%s: Inspecting file '%s'", __func__, entry->d_name);
+
+		std::cmatch match;
+		if (!std::regex_match(entry->d_name, match, magicurlRegex)) {
+			vlog.debug("%s: File '%s' does not match pattern", __func__, entry->d_name);
+			continue;
+		}
+
+		pid_t socketPid = std::stoi(match[1].str());
+		if (socketPid == currentPid) {
+			vlog.debug("%s: Skipping file '%s' (current process PID)", __func__, entry->d_name);
+			continue;
+		}
+
+		std::string filePath = std::string(tmpDir) + "/" + entry->d_name;
+
+		// Check if the file is a socket and belongs to the current user
+		struct stat statBuf;
+		if (stat(filePath.c_str(), &statBuf) == 0 &&
+			S_ISSOCK(statBuf.st_mode) &&
+			statBuf.st_uid == currentUid) {
+			vlog.debug("%s: File '%s' is a socket and owned by the current user", __func__, filePath.c_str());
+
+			// Check if there's someone listening behind the socket
+			int sockFd = socket(AF_UNIX, SOCK_STREAM, 0);
+			if (sockFd == -1) {
+				vlog.error("%s: Failed to create socket: %s", __func__, strerror(errno));
+				continue;
+			}
+
+			struct sockaddr_un addr;
+			memset(&addr, 0, sizeof(addr));
+			addr.sun_family = AF_UNIX;
+			strncpy(addr.sun_path, filePath.c_str(), sizeof(addr.sun_path) - 1);
+
+			if (connect(sockFd, (struct sockaddr*)&addr, sizeof(addr)) == 0) {
+				close(sockFd);
+				closedir(dir);
+				vlog.debug("%s: Successfully connected to socket '%s'", __func__, filePath.c_str());
+				return strdup(filePath.c_str());
+			} else {
+				vlog.debug("%s: Failed to connect to socket '%s': %s", __func__, filePath.c_str(), strerror(errno));
+			}
+
+			close(sockFd);
+		} else {
+			vlog.debug("%s: File '%s' is not a valid socket or not owned by the current user", __func__, filePath.c_str());
+		}
+	}
+
+	closedir(dir);
+	vlog.info("%s: No suitable magicurl socket found", __func__);
+	return NULL;
+}
+#endif /* !defined(WIN32) && !defined(WIN64) && !defined(__APPLE__) */
+
 #if !defined(WIN32) && !defined(WIN64) && !defined(__APPLE__)
 char* MagicUrlHelper::get_magicurl_socket_path() {
 		/* To make it possible that several viewer sessions can be started without using the
@@ -597,7 +678,54 @@ int MagicUrlHelper::sendUrlToViewer(const char* url) {
 	free(urlEncoded);
 	return succeeded;
 #else
-	vlog.info("%s: function not yet implemented for linux!", __func__);
-	return 0;
+	vlog.debug("%s: Trying to send a message via UNIX socket to a possibly running viewer", __func__);
+
+	char* urlEncoded = url_semi_encode(url);
+	if (!urlEncoded) {
+		vlog.error("%s: Failed to encode URL. Can't send URL through UNIX socket", __func__);
+		return 0;
+	}
+
+	vlog.debug("%s: urlEncoded: %s", __func__, urlEncoded);
+
+	const char* socketPath = findOpenMagicurlSocket();
+	if (!socketPath) {
+		vlog.info("%s: didn't find a path to an open socket", __func__);
+		return 0;
+	}
+	vlog.debug("%s: socketPath: %s", __func__, socketPath);
+
+	int sockFd = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (sockFd == -1) {
+		vlog.error("%s: Failed to create socket: %s", __func__, strerror(errno));
+		free(urlEncoded);
+		return 0;
+	}
+
+	struct sockaddr_un serverAddr;
+	memset(&serverAddr, 0, sizeof(serverAddr));
+	serverAddr.sun_family = AF_UNIX;
+	strncpy(serverAddr.sun_path, socketPath, sizeof(serverAddr.sun_path) - 1);
+
+	if (connect(sockFd, (struct sockaddr*)&serverAddr, sizeof(serverAddr)) == -1) {
+		vlog.error("%s: Failed to connect to UNIX socket '%s': %s", __func__, socketPath, strerror(errno));
+		close(sockFd);
+		free(urlEncoded);
+		return 0;
+	}
+
+	ssize_t sentBytes = send(sockFd, urlEncoded, strlen(urlEncoded) + 1, 0); // Send URL including null terminator
+	if (sentBytes == -1) {
+		vlog.error("%s: Failed to send data through UNIX socket: %s", __func__, strerror(errno));
+		close(sockFd);
+		free(urlEncoded);
+		return 0;
+	}
+
+	vlog.debug("%s: Sent (encoded) URL: '%s' through UNIX socket '%s'", __func__, urlEncoded, socketPath);
+
+	close(sockFd);
+	free(urlEncoded);
+	return 1;
 #endif /* WIN32 */
 }
