@@ -284,7 +284,7 @@ namespace NativeMessagingApp {
 		HANDLE hFind = FindFirstFile(searchPath, &findFileData);
 
 		if (hFind == INVALID_HANDLE_VALUE) {
-			vlog.info("%s: Unable to list directories in '%s'", __func__, basePath);
+			vlog.debug("%s: Unable to list directories in '%s'", __func__, basePath);
 			return;
 		}
 
@@ -308,7 +308,7 @@ namespace NativeMessagingApp {
 #else
 		DIR* dir = opendir(basePath);
 		if (!dir) {
-			vlog.info("%s: Unable to list directories in '%s'", __func__, basePath);
+			vlog.debug("%s: Unable to list directories in '%s'", __func__, basePath);
 			return;
 		}
 
@@ -474,7 +474,7 @@ namespace NativeMessagingApp {
 			vlog.error("%s: Unable to create registry key", __func__);
 		}
 #else
-		vlog.info("%s: no registry in Linux :) It's enough to have the manifest files in the right place", __func__);
+		vlog.debug("%s: no registry in Linux :) It's enough to have the manifest files in the right place", __func__);
 #endif /* WIN32 */
 	}
 
@@ -577,7 +577,7 @@ namespace NativeMessagingApp {
 			vlog.error("%s: Failed to get $HOME from env)", __func__);
 			return NULL;
 		}
-		vlog.info("%s: browser %d (%s)", __func__, browser, browserToString(browser));
+		vlog.debug("%s: browser %d (%s)", __func__, browser, browserToString(browser));
 
 		char manifestDir[PATH_MAX];
 		switch (browser) {
@@ -664,17 +664,17 @@ namespace NativeMessagingApp {
 
 	bool registerAsNativeMessagingHostApp(const char* configDir) {
 		char* vncviewerBinaryPath = getVncviewerBinaryPath(); // Something like C:\\Program Files\\TightGate-Pro\\vncviewer.exe for Windows and /usr/local/bin/tightgateviewer for Linux
-		vlog.info("%s: juaaaa. got vncviewerBinaryPath: %s", __func__, vncviewerBinaryPath);
+		vlog.debug("%s: Got vncviewerBinaryPath: %s", __func__, vncviewerBinaryPath);
 		for (Browser browser = BROWSER_FIREFOX; browser < BROWSER_COUNT; browser = static_cast<Browser>(static_cast<int>(browser) + 1)) {
 			char* allowedJsonArray = buildAllowedJsonArray(browser);
 			if (allowedJsonArray && *allowedJsonArray) {
-				vlog.info("%s: Created 'allowed' JSON array for browser %d: %s", __func__, browser, allowedJsonArray);
+				vlog.debug("%s: Created 'allowed' JSON array for browser %d: %s", __func__, browser, allowedJsonArray);
 			} else {
 				vlog.error("%s: Failed to create an 'allowed' JSON array for browser (%d)", __func__, browser);
 			}
 			char* manifestPath = getManifestPath(configDir, browser);
 			if (manifestPath) {
-				vlog.info("%s: Manifest path for browser %d: %s", __func__, browser, manifestPath);
+				vlog.debug("%s: Manifest path for browser %d: %s", __func__, browser, manifestPath);
 			} else {
 				vlog.error("%s: Failed to create a manifest path for browser (%d)", __func__, browser);
 			}
