@@ -1,6 +1,6 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
  * Copyright 2011-2019 Pierre Ossman for Cendio AB
- * Copyright (C) 2014-2023 m-privacy GmbH
+ * Copyright (C) 2014-2025 m-privacy GmbH
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -791,7 +791,13 @@ void Viewport::handleClipboardChange(int source, void *data)
     vlog.debug("Local clipboard changed whilst not focused, will notify server later");
     self->pendingClientClipboard = true;
     // Clear any older client clipboard from the server
-    self->cc->announceClipboard(false);
+    try {
+      self->cc->announceClipboard(false);
+    } catch (rdr::Exception& e) {
+      vlog.error("%s", e.str());
+      exit_vncviewer(_("An unexpected error occurred when communicating "
+                       "with the server:\n\n%s"), e.str());
+    }
     return;
   }
 
@@ -1368,7 +1374,7 @@ void Viewport::popupContextMenu()
     window()->cursor(FL_CURSOR_DEFAULT);
 
   // FLTK also doesn't switch focus properly for menus
-  handle(FL_UNFOCUS);
+  Fl::handle(FL_UNFOCUS, window());
 
 #ifdef WIN32
   if(GetCursorPos(&cursorPos))
@@ -1377,7 +1383,7 @@ void Viewport::popupContextMenu()
 
   m = contextMenu->popup();
 
-  handle(FL_FOCUS);
+  Fl::handle(FL_FOCUS, window());
 
   // Back to our proper mouse pointer.
   if (Fl::belowmouse())
